//
// C++ Interface: channelData.h
//
// Description:
//
//
// Author: RealQuick <RealQuick@doubleQuick.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef __CHANNELDATA_H_
#define __CHANNELDATA_H_

#ifdef USE_TINY_XML
#include "tinyxml.h"
#endif

#include <vdr/epg.h>

#include <iostream>
#include <time.h>
#include <string>
#include <vector>
#include <libsi/si.h>

#include "misc.h"

#include "common.h"

using namespace std;

struct programmeData
{
    /* Title, start and end times and a description of the programme. */
    string title;
    string start_time;
    string end_time;
    string description;
    string short_text;
    string infourl;
    bool update;
    string genre;
    string country;
    string actor;
    string year;
    string regisseur;
    string moderator;
    string studio_guest;
    tChannelID ChannelID;

    string vps;
    string toptip;
    bool live;
    bool coded;
    bool age_marker;
    bool dolby;
    bool stereo;
    int sequence;
    bool wide;
    bool primetime;

    int region_id;

    long eventid;

    string jpeg;
    string png;

    string date;	// for provider bleb.org
    string id;
    string source;
};                              // end struct programmeData

class cChannelData
{
private:
#ifdef USE_TINY_XML
    TiXmlDocument doc;
#endif

    string EpgDir;
    string EpgDataFileName;

    bool isLoaded;

    int m_ProviderType;

    const char *tipdesc[4];

    string id;
    string source;
    string date;

    std::vector < programmeData > ProgrammeList;

public:

    cChannelData (const char *dir, const char *epgFileName, int ProviderType);
    ~cChannelData ();

    int loadEpgData (cProviderChannelList *ChannelList, int nChannelID = -1);
    bool parseEpgDataFile (cProviderChannelList *ChannelList);

#ifdef USE_TINY_XML
    int getChannelData_Prov0 (TiXmlNode * /* , cChannelData * */); // extracts data from tinyXML data structure to cChannelData
    int getChannelData_Prov1 (TiXmlNode *, /* cChannelData * , */ cProviderChannelList *ChannelList); // extracts data from tinyXML
#endif
    int getChannelData_Prov1 (cProviderChannelList *ChannelList); // extracts data sequential from xml file
};                              // end class cChannelData

class cCleanChannelData : public cThread
{
private:
    cSchedule *m_Schedule;
    tChannelID m_cid;
    int *m_Reverse_ChannelList;
public:
    cCleanChannelData(cSchedule *Schedule, tChannelID cid = tChannelID(), int *Reverse_ChannelList = NULL);

    int ClearEvents(cSchedule *Schedule, tChannelID cid = tChannelID());	// Remove duplicate events (from dvb stream)

    int CleanNow(bool bThread = false);

    void Action ();
};

#endif
