/*
*  C Implementation: channelmap.c
*
* Description:  Classes used for the channel mapping vdr - provider
*
* Author: RealQuick <RealQuick@doubleQuick.de>, (C) 2009
*
* Copyright: See COPYING file that comes with this distribution
*
*/
#include <vector>
#include <map>
#include <string>
#include <vdr/plugin.h>
#include <vdr/device.h>
#include <vdr/i18n.h>
#include "i18n.h"

#include "common.h"
#include "channelmap.h"
#include <string>
#include "vdr/remote.h"

#include "setup.h"

#include "misc.h"

#define CHNUMWIDTH  (numdigits(Channels.MaxNumber()) + 1)

extern const char *providerNames[];
extern std::vector < cProviderStatus > Providers;   // list of providers

using namespace std;

const ChannelData *cProviderChannels::FirstChannel(void)
{
    vpos = 0;
    return &ProviderChannelList.front();
};

const ChannelData *cProviderChannels::NextChannel(void)
{
    if (vpos < (ProviderChannelList.size() - 1))
        return &ProviderChannelList.at(++vpos);
    else
        return NULL;
};

const ChannelData *cProviderChannels::ChannelById(int id)
{
    for (unsigned int x=0;x<ProviderChannelList.size();x++)
    {
        if (ProviderChannelList[x].nHashIndex == id) // nChannel
        {
            return &ProviderChannelList[x];
        }
    }
    return NULL;
}

cProviderChannels::cProviderChannels(cProviderChannelList *ChannelList)
{
    ChannelData itm;
    vpos = 0;

    FILE *fh;

    char sFile[256];
    char sText[1024];
    string sBuffer;

    ProviderChannelList.clear();

    sprintf(sFile, "%s/%s", cPlugin::ConfigDirectory ( PLUGIN_NAME_I18N ), CHANNELS_ALL_CONF);
    fh = fopen(sFile, "r");

    if (fh == NULL)
    {
        Skins.QueueMessage ( mtError, tr ( "Error while opening file with available provider channels" ), 4 );
        esyslog("iepg: can not open file %s", sFile);
        return;
    }

    int nIndex = 0;
    while ( !feof ( fh ) )
    {
        if ( fgets ( sText , sizeof(sText), fh ) != NULL && strlen(sText) && strlen(sText))
        {
            sBuffer = sText;
            unsigned int pos1, pos2;
            pos1 = sBuffer.find ( "|" );
            if ( pos1 !=string::npos )
            {
                pos2 = sBuffer.find ( "|", pos1 + 1 );
                if ( pos2 !=string::npos )
                {
//                    dsyslog("iepg: [%s] nPos1=%d, nPos2=%d", sText, pos1, pos2);
//                    dsyslog("iepg: nPos1=%d, nPos2=%d, Provider=%s, nChannel=%s sChannel=%s", pos1, pos2, sBuffer.substr ( 0, pos1 ).c_str(), sBuffer.substr ( pos2 + 1).c_str(), sBuffer.substr ( pos1 + 1, pos2 - pos1 - 1).c_str());
                    itm.nProvider = atoi(sBuffer.substr ( 0, pos1 ).c_str());
                    itm.nChannel = atoi(sBuffer.substr ( pos2 + 1).c_str());
                    itm.sChannel = sBuffer.substr ( pos1 + 1, pos2 - pos1 - 1);
                    if (itm.nChannel==0)
                    {
                        itm.sUrl = sBuffer.substr ( pos2 + 1).c_str();

                        unsigned int nPos = sBuffer.rfind('/', -1);
                        if (nPos !=string::npos)
                        {
                            string sc = sBuffer.substr(nPos+1);
                            sc = sc.substr(0, sc.length() - 1);
                            SetChannelData ( ChannelList, itm.sChannel.c_str(), &itm.nChannel, (char *)sc.c_str(), itm.nProvider );
                            itm.nHashIndex = itm.nChannel;	// Computed hashindex returned from SetChannelData is index
                        }
                        else
                        {
                            itm.nChannel = 0;
                            esyslog("iepg:  / is missing in URL (%s)", itm.sUrl.c_str());
                        }
                    }
                    else
                    {
                        // channelid will be replaced with the hashvalue in SetChannelData
                        itm.nHashIndex = itm.nChannel;
                        SetChannelData ( ChannelList, itm.sChannel.c_str(), &itm.nHashIndex, "", itm.nProvider );
                    }
                    itm.nPos = ++nIndex;
                    ProviderChannelList.push_back(itm);
                }
            }

        }
    }

    fclose(fh);
    dsyslog("iepg: file %s was successfully read", sFile);

}

cProviderChannels::~cProviderChannels()
{
    ProviderChannelList.clear();
}


void cMenuChannelItemMapping::Set()
{
    char *buffer = NULL;
    if (!channel->GroupSep())
    {
        int nMap = Reverse_ChannelList[channel->Number()];
        string sMap;

        if (nMap == 0)
            sMap = tr("<*DVB*>");
        else
        {
            const ChannelData *x = ProviderChannels->ChannelById( nMap );
            if (x != NULL)
            {
                sMap = providerNames[x->nProvider];	// Get provider name from global array
                sMap += "\t";
                sMap += x->sChannel;
            }
            else
            {
                sMap = "Mapping for hashindex ";
                sMap += itoa(nMap);
                sMap += " failed!";
            }
        }

        asprintf(&buffer, "%d\t%s\t%s", channel->Number(), channel->Name(), sMap.c_str());


    }
    else
        asprintf(&buffer, "---\t%s ----------------------------------------------------------------", channel->Name());

    SetText(buffer, true);
    free(buffer);
}

eOSState cMenuChannelItemMapping::ProcessKey(eKeys Key)
{
    eOSState state = cOsdItem::ProcessKey(Key);

    switch (state)
    {
    default:
        if (state == osUnknown)
        {
            switch (Key)
            {
            case kOk:
                *nChannelSelected = nChannelNumber;	// save selected Channel number and give it to the submenu (in kOK - cMenuChannelMapping)
                break;
            case kYellow:
                break;
            case kRed:
            case kGreen:
                break;
            default:
                break;
            }
        }
    }
    return state;
}


cMenuChannelsMapping::cMenuChannelsMapping(void)
        :cOsdMenu(tr("channel map"), CHNUMWIDTH, 20, 10, 10)
{
    // create a main channel group
    mainGroup.Parse(tr(":Main channels"));

    ChannelList = new cProviderChannelList[MAX_PROVIDER_CHANNEL + PROVIDER_HASHSIZE + COLLISION_TABSIZE];
    Reverse_ChannelList = new int[MAX_VDR_CHANNELS];

    ProviderChannels = new cProviderChannels(ChannelList);

    for (int p=0;p<(MAX_PROVIDER_CHANNEL + PROVIDER_HASHSIZE + COLLISION_TABSIZE);p++)
    {
        ChannelList[p].ChannelId = tChannelID();
        ChannelList[p].Provider=-1;
    }
    for (int c=0;c<MAX_VDR_CHANNELS;c++) Reverse_ChannelList[c] = 0;

    Urls.clear();
    ReadUrlList ( NULL, &Urls, (char *)cPlugin::ConfigDirectory ( PLUGIN_NAME_I18N ), /* nProvider=*/ 0, ChannelList, /* Download=*/false);
    Urls.clear();
    ReadUrlList ( NULL, &Urls, (char *)cPlugin::ConfigDirectory ( PLUGIN_NAME_I18N ), /* nProvider=*/ 1, ChannelList, /* Download=*/false);

    for (int x=0;x < (MAX_PROVIDER_CHANNEL + PROVIDER_HASHSIZE + COLLISION_TABSIZE);x++)
    {
        if (ChannelList[x].ChannelId.Source() != tChannelID().Source())
        {
            cChannel *chan = Channels.GetByChannelID(ChannelList[x].ChannelId);
            if (chan->Number()<0 || chan->Number() >= MAX_VDR_CHANNELS)
            {
                esyslog("iepg: setup: channel number out of range (%d)", chan->Number());
                Skins.QueueMessage ( mtError, tr ("channel number out of range!"), 4);
                return;
            }
            Reverse_ChannelList[chan->Number()] = x;
        }
    }

    dsyslog("iepg: reverse channel list constructed");

    Set();
    SetHelp(tr("Save"), NULL, NULL, NULL);
}

cMenuChannelsMapping::~cMenuChannelsMapping()
{
    if (ChannelList != NULL)
        delete [] ChannelList;  // Cleanup stack
    if (Reverse_ChannelList != NULL)
        delete [] Reverse_ChannelList;  // Cleanup stack
    if (ProviderChannels != NULL)
        delete ProviderChannels;

}

void cMenuChannelsMapping::SetChannelMode()
{
    FILE *fh = NULL;

    string sPath = cPlugin::ConfigDirectory ();
    int nPos = sPath.rfind("/plugins"); sPath.erase(nPos + 1);

    fh = fopen ( (sPath + "noepgchannels.conf").c_str(), "w" );
    if ( fh == NULL )
    {
        Skins.QueueMessage ( mtError, tr ( "Error while opening config file" ), 4 );
        esyslog ( "iepg: can not open file %s for writing", sPath.c_str() );
    }

    time_t now = time(NULL);
    struct tm *ts;
    ts = localtime(&now);
    
    fputs ( "#\n# noepgchannels.conf\n#\n", fh );
    char *p; asprintf(&p, "# Generated by iepg plugin: %s#\n# Note: Manually changes in this file may be overwritten\n#\n", asctime(ts));
    fputs ( p, fh );
    free(p);

    std::string chList;
    for ( int x=0;x < MAX_VDR_CHANNELS;x++ )
    {
        int r = Reverse_ChannelList[x];
        if (r)
        {
            // get provider from ProviderChannelList
            cChannel *ch = Channels.GetByNumber ( x );
//            chList += std::string(chList.empty()?"":" ") + *ch->GetChannelID().ToString();

	    {
	        chList = "-";
		chList += *ch->GetChannelID().ToString();
		chList += "\n";

	        fputs ( chList.c_str(), fh );
	    }
        }
    }

    fclose(fh);

    // Force syncing of channels
    Channels.Load((sPath + "channels.conf").c_str());

#ifdef NOEPG_PATCH
    dsyslog("iepg: set noEPGList to %s", chList.c_str());
    if (::Setup.noEPGList) free(::Setup.noEPGList); 
    ::Setup.noEPGList = strdup(chList.c_str());
    ::Setup.Save();
#endif
}

bool cMenuChannelsMapping::SaveSettings()
{
    FILE *fh = NULL;

    char sFile[512];
    bool first_time = true;

    for ( unsigned int i=0;i < Providers.size();i++ )
    {
//        if ( iepgSetup.SetupProviders[i].ProviderType == 1 ) // At the moment only provider type 1 is saved
        {
            sprintf ( sFile, "%s/%d/%s", cPlugin::ConfigDirectory ( PLUGIN_NAME_I18N ), i, URLLISTFILENAME );
            fh = fopen ( sFile, "w" );

            if ( fh == NULL )
            {
                Skins.QueueMessage ( mtError, tr ( "Error while opening config file" ), 4 );
                esyslog ( "iepg: can not open file %s", sFile );
                return false;
            }

            for ( int x=0;x < MAX_VDR_CHANNELS;x++ )
            {
                unsigned int r = Reverse_ChannelList[x];
                if (r)
                {
                    // get provider from ProviderChannelList
                    const ChannelData *pc = ProviderChannels->ChannelById(r);
                    if (pc)
                    {
                        ChannelList[r].Provider = pc->nProvider;

                        if (ChannelList[r].Provider == (int)i)
                        {
                            cChannel *chan = Channels.GetByNumber ( x );
                            string sOut=chan->Name();

                            switch (ChannelList[r].Provider)
                            {
                            case 0:
                                sOut += ";";
                                sOut += iepgSetup.SetupProviders[i].baseurl;
                                sOut += "/";
                                sOut += ChannelList[r].ChannelName;
                                break;

                            case 1:
                                sOut += "|";
                                sOut += itoa(pc->nChannel);
                                sOut += ";";

                                if ( first_time )
                                {
                                    sOut += iepgSetup.SetupProviders[i].baseurl;
                                    first_time = false;
                                }
                                break;

                            }

                            sOut += "\n";
                            fputs ( sOut.c_str(), fh );
                        }
                    }
                }
            }
            fclose ( fh );
        }
    }

    // if NOEPG-patched vdr -> build List and update setup.conf
    if (iepgSetup.NoEPGWorkaround == NOEPG_PATCHED_VDR)
        SetChannelMode();

//    fclose ( fh );
    return true;
}

void cMenuChannelsMapping::Set(void)
{
    const cChannel *currentChannel = GetChannel(Current());
    Clear();
    if (Channels.First() && !Channels.First()->GroupSep()) // add a main group if there's none
        Add(new cMenuChannelItemMapping(&mainGroup, Reverse_ChannelList, ProviderChannels, 0, &nChannelSelected));
    for (const cChannel *channel = Channels.First(); channel; channel = Channels.Next(channel))
        if ( *channel->Name())
            Add(new cMenuChannelItemMapping(channel, Reverse_ChannelList, ProviderChannels, channel->Number(), &nChannelSelected), channel == currentChannel);
}

const cChannel *cMenuChannelsMapping::GetChannel(int Index)
{
    cMenuChannelItemMapping *p = (cMenuChannelItemMapping *)Get(Index);
    return p ? p->Channel() : NULL;
}


eOSState cMenuChannelsMapping::ProcessKey(eKeys Key)
{
    eOSState state = cOsdMenu::ProcessKey(Key);

    switch (state)
    {
    default:
        if (state == osUnknown)
        {
            switch (Key)
            {
            case kOk:

                if (nChannelSelected < 0 || nChannelSelected > (MAX_PROVIDER_CHANNEL + PROVIDER_HASHSIZE + COLLISION_TABSIZE))
                {
                    Skins.Message(mtError, "Channel NOT set or out of range!");
                    return state;
                }
                return AddSubMenu(new cMenuProviderChannelsMapping(nChannelSelected, ProviderChannels, Reverse_ChannelList));
                break;

            case kYellow:
                break;
            case kRed:
                if (SaveSettings())
                    Skins.Message(mtInfo, tr("configuration successfully saved"), true);
                else
                    Skins.Message(mtError, tr("Error while saving configuration"), true);
                break;
		state = osContinue;
            case kGreen:
                break;
            default:
                break;
            }
        }
    }
    return state;
}

// menu for provider channel

cMenuProviderChannelItemMapping::cMenuProviderChannelItemMapping(int xChannelNumber, const ChannelData *data, int *xReverse_ChannelList) /* char *sText) */ /* : cOsdItem(sText) */
{

    if (data->nPos)
    {
        string sOut = providerNames[data->nProvider];
        sOut += "\t";
        sOut += data->sChannel.c_str();
        SetText(sOut.c_str(), true);
    }
    else
        SetText(data->sChannel.c_str(), true);  // Reset to DVB as a choice - dummy channel


    Reverse_ChannelList = xReverse_ChannelList;

    nProvider = data->nProvider;

    nChannel = data->nChannel;
    nHashIndex = data->nHashIndex;

    sChannel = "";
    sChannel += data->sChannel;	// No pointer assignment - make copy

    nChannelNumber = xChannelNumber;

}

eOSState cMenuProviderChannelItemMapping::ProcessKey(eKeys Key)
{
    eOSState state = cOsdItem::ProcessKey(Key);

    switch (state)
    {
    default:
        if (state == osUnknown)
        {
            switch (Key)
            {
            case kOk:
                Reverse_ChannelList[nChannelNumber] = nHashIndex;
//                return osBack;
		state = osContinue;
 		cRemote::Put(kBack); 
		break;
            case kYellow:
                break;
            case kRed:
            case kGreen:
                break;
            default:
                break;
            }
        }
    }
    return state;
}

cMenuProviderChannelsMapping::cMenuProviderChannelsMapping(int xChannelNumber, cProviderChannels *xProviderChannels, int *xReverse_ChannelList)
        :cOsdMenu(tr("Please choose from provider channels"), 20, 10, 10)
{
    ProviderChannels = xProviderChannels;
    Reverse_ChannelList = xReverse_ChannelList;

    nChannelNumber = xChannelNumber;	// Channel pos from main menu

    Set();
}


eOSState cMenuProviderChannelsMapping::ProcessKey(eKeys Key)
{
    eOSState state = cOsdMenu::ProcessKey(Key);

    switch (state)
    {
    default:
        if (state == osUnknown)
        {
            switch (Key)
            {
            case kOk:
            case kYellow:
                break;
            case kRed:
                break;
            case kGreen:
                break;
            default:
                break;
            }
        }
    }
    return state;
}

void cMenuProviderChannelsMapping::Set(void)
{
    Clear();

    const ChannelData *clist;

    ChannelData UseDVB;
    UseDVB.nProvider = 0;
    UseDVB.nPos = 0;
    UseDVB.nChannel = 0;
    UseDVB.nHashIndex = 0;
    UseDVB.sChannel = "<* Use epg data from DVB stream *>";
    Add(new cMenuProviderChannelItemMapping(nChannelNumber, &UseDVB, Reverse_ChannelList));

    for (clist=ProviderChannels->FirstChannel(); clist; clist = ProviderChannels->NextChannel())
    {
        if (clist)
        {
            Add(new cMenuProviderChannelItemMapping(nChannelNumber, clist, Reverse_ChannelList));
        }
    }
}

