//
// C++ Interface: channelmap
//
// Description: Classes used for the channel mapping vdr - provider
//
//
// Author: RealQuick <RealQuick@doubleQuick.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef __CHANNELMAP_H
#define __CHANNELMAP_H


#include <string>
#include <vector>
#include <vdr/channels.h>       // for Channels
#include "xmlepg2vdrthread.h"

using namespace std;

struct ChannelData
{
    int nProvider;
    int nPos;
    int nChannel;
    int nHashIndex;
    string sChannel;
    string sUrl;
};

class cProviderChannels
{
private:
    std::vector < ChannelData > ProviderChannelList;
    unsigned int vpos;
public:
    cProviderChannels(cProviderChannelList *ChannelList);
    const ChannelData *FirstChannel(void);
    const ChannelData *NextChannel(void);
    const ChannelData *ChannelById(int id);
    virtual ~cProviderChannels();
};

// --- cMenuChannelItemMapping ----------------------------------------------
class cMenuChannelItemMapping : public cOsdItem
{
private:
    const cChannel *channel;
    int *Reverse_ChannelList;
    cProviderChannels *ProviderChannels;
    int nChannelNumber;
public:
    cMenuChannelItemMapping(const cChannel *Channel, int *xReverse_ChannelList, cProviderChannels *xProviderChannels, int xChannelNumber, int *xChannelSelected) : channel(Channel)
    {
        Reverse_ChannelList = xReverse_ChannelList;
        ProviderChannels=xProviderChannels;
        nChannelNumber=xChannelNumber;
        nChannelSelected=xChannelSelected;
        Set();
    }
    void Set();
    const cChannel *Channel(void)
    {
        return channel;
    }

    int *nChannelSelected;	// Pointer to variable in cMenuChannelsMapping Updated on kOK
    virtual eOSState ProcessKey(eKeys Key);
};

// --- cMenuChannelsMapping ---------------------------------------------------------
class cMenuChannelsMapping : public cOsdMenu
{
private:
    void Set(void);
    const cChannel *GetChannel(int Index);
    cChannel mainGroup;

    cProviderChannels *ProviderChannels;

    int *Reverse_ChannelList;	//TODO Move Reverse_ChannelList  -> iepg.c

    cProviderChannelList *ChannelList;
    std::vector < cURLFile > Urls;
    cURLFile tmp_obj;

    int nChannelSelected;

    const char *providerchannels[500];
public:
    cMenuChannelsMapping(void);
    virtual eOSState ProcessKey(eKeys Key);
    virtual ~cMenuChannelsMapping();

    void SetChannelMode();
    bool SaveSettings();
};

// menu for provider channel

// --- cMenuProviderChannelItemMapping ----------------------------------------------
class cMenuProviderChannelItemMapping : public cOsdItem
{
private:
    int nChannelNumber;
    int nProvider;
    int nChannel;
    string sChannel;
    string sBuffer;

    int nHashIndex;

    int *Reverse_ChannelList;
public:
    cMenuProviderChannelItemMapping(int xChannelNumber, const ChannelData *data, int *xReverse_ChannelList);
    virtual eOSState ProcessKey(eKeys Key);
};

// --- cMenuProviderChannelsMapping ---------------------------------------------------------
class cMenuProviderChannelsMapping : public cOsdMenu
{
private:
    void Set(void);
    cProviderChannels *ProviderChannels;
    int *Reverse_ChannelList;
    int nChannelNumber;
public:
    cMenuProviderChannelsMapping(int xChannelNumber, cProviderChannels *xProviderChannels, int *xReverse_ChannelList);
    virtual eOSState ProcessKey(eKeys Key);
};

#endif

