//
// C++ Interface: iepg.c
//
// Description:
//
//
// Author: RealQuick <RealQuick@doubleQuick.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <vdr/plugin.h>         // for cPlugin
// #include <vdr/interface.h>      // for Interface->Confirm
// #include <vdr/osdbase.h>        // for cOsdItem
#include <time.h>               // for time_t
#include "iepg.h"
#include "setup.h"
#include "menu.h"
#include "i18n.h"
#include "xmlepg2vdrthread.h"   // all work of plugin in cThreadXmlEpg2vdr::Action()

#include <getopt.h>

#include "channelData.h"
#include "channelmap.h"
#include "setup.h"

static const char *VERSION = "0.1.1-3";
static const char *DESCRIPTION = trNOOP("A plugin to import EPG data from internet into VDR");
static const char *MAINMENUENTRY = tr("EPG via Internet");

// Globals
cThreadXmlEpg2vdr *ThreadXml2vdr;

cStatusRecording *CheckRecording;
cStatusChannelSwitch *CheckChannelSwitch;

int *Reverse_ChannelList = NULL;

time_t lastRun;
time_t StartupTime = 0;

// #define FREQRUN (60*60*12)

/**** Add provider names here ******/

const char *providerNames[] =
{
    tr("bleb.org"),
    tr("epgdata.com"),
    //tr("EPG from Provider 1"),
    //tr("EPG from Provider 2"),
    NULL
};
// Indices are used, so changing order of providers should be done with caution


std::vector < cProviderStatus > Providers;   // list of providers

/***************** class cPluginIEpg *****************************
 * This class creates an object of cThreadXmlEpg2vdr.
 * And initiates a thread by calling cThreadXmlEpg2vdr::Start(). All work is done in this thread.
 */

const char *cPluginIEpg::Version (void)
{
    return VERSION;
}
const char *cPluginIEpg::Description (void)
{
    return DESCRIPTION;
}


cPluginIEpg::cPluginIEpg (void)
{
    // Initialize any member variables here.
    // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
    // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!

    ThreadXml2vdr = NULL;

    CheckRecording = NULL;
    CheckChannelSwitch = NULL;
    sDir = NULL;
    /*
     * initialize the Provider vector
     */
    cProviderStatus p;
    cProviderStatus s;

    // Default pathes Can be overwritten due command line options
    iepgSetup.sImageDirectory = IMAGE_PATH;
    iepgSetup.sDataDirectory = IEPG_PATH;

    Providers.clear();
    iepgSetup.SetupProviders.clear();

    for (int i = 0; providerNames[i] != NULL; i++) // Change this number
    {
        p.ProviderType = i;
        p.Name = providerNames[i];
        Providers.push_back (p);

        // Initialize vector for setup-struct
        s.Name = providerNames[i];
        iepgSetup.SetupProviders.push_back(s);
    }

}

cPluginIEpg::~cPluginIEpg ()
{
    if (CheckRecording != NULL)
        delete CheckRecording;

    if (CheckChannelSwitch != NULL)
        delete CheckChannelSwitch;

    if (ThreadXml2vdr != NULL)
    {
        delete ThreadXml2vdr;
    }

    if (Reverse_ChannelList != NULL)
        delete [] Reverse_ChannelList;

    if (!sDir)
        free(sDir);
}

const char *
cPluginIEpg::MainMenuEntry(void) 
{ 
    return MAINMENUENTRY; 
}

const char * cPluginIEpg::CommandLineHelp (void)
{
    // Return a string that describes all known command line options.
    return " -i epg_image_folder -d temporary_data_folder\n";}

bool cPluginIEpg::ProcessArgs (int argc, char *argv[])
{
    static struct option long_options[] = {
		{ "temporary_data_folder", optional_argument, NULL, 'd' },
		{ "epgimagefolder", optional_argument, NULL, 'i' },
		{ NULL }};
    bool retval=true;
    int c;
    while ((c = getopt_long(argc, argv, "d:i:", long_options, NULL)) != -1) {
	switch (c) {
	    case 'd':
		if(strlen(optarg))
		     iepgSetup.sDataDirectory = optarg;
		break;
	    case 'i':
		if(strlen(optarg))
		    iepgSetup.sImageDirectory=optarg;
		break;
	    default:  
		break;
	} // switch
  } // while
  return retval;
}

bool
cPluginIEpg::Initialize (void)
{
    // Initialize any background activities the plugin shall perform.
    RegisterI18n(tlPhrases);
    return true;
}

bool
cPluginIEpg::Start (void)
{
    StartupTime = time(NULL);	// remeber Startup time
    dsyslog("iepg: set startup time to %ld", StartupTime);

    if (iepgSetup.hour_to_start == IMPORT_AFTER_STARTUP)
    {
        t_Start = StartupTime + WAIT_AFTER_STARTUP;
        dsyslog("iepg: wait %d minutes before starting import", WAIT_AFTER_STARTUP / 60);
    }
    // values > 1 means start import at (value - 2) clock
    if (iepgSetup.hour_to_start > IMPORT_AFTER_STARTUP)
    {
        time_t t1 = time(NULL);
        struct tm *tm=localtime ( &t1 );

        // specified hour is over -> setup wakeup time to next day
        if (tm->tm_hour >= (iepgSetup.hour_to_start - 1))
        {
            tm->tm_hour = iepgSetup.hour_to_start - 1;
            tm->tm_min = 0;
            tm->tm_sec = 0;
            t1 = mktime ( tm ) + ( time_t ) ( 24 * 60 * 60);
        }
        else
        {
            tm->tm_hour = iepgSetup.hour_to_start - 1;
            tm->tm_min = 0;
            tm->tm_sec = 0;
            t1 = mktime ( tm );
        }
        t_Start = t1;

        struct tm *ts;
        ts = localtime(&t_Start);
        dsyslog("iepg: set startup time of import to %s", asctime(ts));
    }

    if (!sDir)
        sDir = strdup(cPlugin::ConfigDirectory ( PLUGIN_NAME_I18N ));

    if (ThreadXml2vdr != NULL)
    {
        delete ThreadXml2vdr;
    }
    ThreadXml2vdr = new cThreadXmlEpg2vdr(sDir); // global object - supply copy of ConfigDirectory

    // for NoEPGWorkaround types 1,2,3 reverse channel list are needed
    // if (iepgSetup.NoEPGWorkaround && iepgSetup.NoEPGWorkaround != NOEPG_PATCHED_VDR)
    if (iepgSetup.NoEPGWorkaround /* && iepgSetup.NoEPGWorkaround != NOEPG_PATCHED_VDR */)
    {

// Construct reverse channel list (purpose: Find provider channels mapped to vdr channels)
        if (!Reverse_ChannelList)
            Reverse_ChannelList = new int[MAX_VDR_CHANNELS];

        cProviderChannelList *ChannelList = new cProviderChannelList[MAX_PROVIDER_CHANNEL + PROVIDER_HASHSIZE + COLLISION_TABSIZE];

//  ProviderChannels = new cProviderChannels(ChannelList);

        for (int p=0;p<(MAX_PROVIDER_CHANNEL + PROVIDER_HASHSIZE + COLLISION_TABSIZE);p++) ChannelList[p].ChannelId = tChannelID();
        for (int c=0;c<MAX_VDR_CHANNELS;c++) Reverse_ChannelList[c] = 0;

        std::vector < cURLFile > Urls;

        ReadUrlList ( NULL, &Urls, sDir, /* nProvider=*/ 0, ChannelList, /* Download=*/false);
        Urls.clear();
        ReadUrlList ( NULL, &Urls, sDir, /* nProvider=*/ 1, ChannelList, /* Download=*/false);

        for (int x=0;x < (MAX_PROVIDER_CHANNEL + PROVIDER_HASHSIZE + COLLISION_TABSIZE);x++)
        {
            if (ChannelList[x].ChannelId.Source() != tChannelID().Source())
            {
                cChannel *chan = Channels.GetByChannelID(ChannelList[x].ChannelId);
                if (chan->Number()<0 || chan->Number() >= MAX_VDR_CHANNELS)
                {
                    esyslog("iepg: setup: channel number out of range (%d)", chan->Number());
                    Skins.QueueMessage ( mtError, tr ("channel number out of range!"), 4);
//              return;
                }
                Reverse_ChannelList[chan->Number()] = x;
            }
        }

        delete [] ChannelList;

        dsyslog("iepg: reverse channel list constructed");

        cCleanChannelData Metzger(NULL, tChannelID() /* chn->GetChannelID() */, Reverse_ChannelList);
        Metzger.CleanNow(false);

        dsyslog("iepg: initial cleanup of duplicate EPG events has finished");

        if (iepgSetup.NoEPGWorkaround == NOEPG_CHANNELSWITCH || iepgSetup.NoEPGWorkaround == NOEPG_CHANNELSWITCH_HOUSEKEEPING)
        {
            CheckChannelSwitch = new cStatusChannelSwitch();
            CheckChannelSwitch->Start();
        }
    }
    else
        dsyslog("iepg: NoEPGWorkaround not set");

    if (iepgSetup.save_rec_pic && CheckRecording == NULL)
    {
        CheckRecording = new cStatusRecording;
        CheckRecording->Start();
    };

    return true;
}




void cPluginIEpg::Stop (void)
{
    StartupTime = 0;	// remeber Startup time
    dsyslog("iepg: reset startup time");

    // Stop any background activities the plugin shall perform.
    ThreadXml2vdr->killThread (0); // calls cThread::Cancel()
}

void cPluginIEpg::Housekeeping (void)
{

    // Thread is running -> wait until next trigger
    if (ThreadXml2vdr->isRunning())
    {
        dsyslog("iepg: Housekeeping : Thread is busy -> ignore");
        return;
    }

    // set startup time if import after system start is set
    if (iepgSetup.hour_to_start && time(NULL) > t_Start)
    {
        if (ThreadXml2vdr)
            ThreadXml2vdr->Start();

        if (iepgSetup.hour_to_start > IMPORT_AFTER_STARTUP)
            t_Start += (time_t)(24 * 60 * 60);	// set start time for next day
        else
            t_Start += (time_t)(365 * 24 * 60 * 60);	// set start time to next year :) (do not repeat)

        return;	// Dont do channel clean cause it is already done within normal import
    }


#if 1
    if (iepgSetup.NoEPGWorkaround == NOEPG_HOUSEKEEPING || iepgSetup.NoEPGWorkaround == NOEPG_CHANNELSWITCH_HOUSEKEEPING)
    {
        cCleanChannelData Metzger(NULL, tChannelID() /* chn->GetChannelID() */, Reverse_ChannelList);
//        Metzger.Start();
        Metzger.CleanNow(false);
    }
#endif


}


cString cPluginIEpg::Active (void)
{
    // Return a message string if shutdown should be postponed
    if (ThreadXml2vdr)
        if (ThreadXml2vdr->isRunning())
            return tr("iepg is currently downloading epg data");
    return NULL;
}

cOsdObject * cPluginIEpg::MainMenuAction (void)
{
    // Return a setup menu in case the plugin supports one.
    return new ciepgMenu(sDir);
}


cMenuSetupPage *cPluginIEpg::SetupMenu(void)
{
    // Return a setup menu in case the plugin supports one.
    return new ciepgMenuSetupPage;
}


bool cPluginIEpg::SetupParse(const char *Name, const char *Value)
{
    int nIndex;
    unsigned char byte;

    nIndex = 0;
    if (strlen(Name) > 0)
    {
        byte = Name[strlen(Name)-1];
        if (byte >= 0x30 && byte <= 0x39)
            nIndex = byte - 0x30;
    }

    if (!strcasecmp(Name, "UseProxy"))    iepgSetup.useproxy = atoi(Value);
    else if (!strcasecmp(Name, "HTTPProxy"))   strcpy(iepgSetup.httpproxy, Value);
    else if (!strcasecmp(Name, "StartupTime"))  iepgSetup.hour_to_start = atoi(Value);
    else if (!strcasecmp(Name, "LastUpdate"))  iepgSetup.lastupdate = atol(Value);
    else if (!strcasecmp(Name, "SavePictureRecordings"))  iepgSetup.save_rec_pic = atoi(Value);
    else if (!strcasecmp(Name, "ConvertPng"))  iepgSetup.convert_to_png = atoi(Value);
    else if (!strcasecmp(Name, "ConvertRaw"))  iepgSetup.convert_to_raw = atoi(Value);
    else if (!strcasecmp(Name, "Priority"))  iepgSetup.Priority = atoi(Value);
    else if (!strcasecmp(Name, "PicturesPrimeTime"))  iepgSetup.pics_only_primetime = atoi(Value);
    else if (!strcasecmp(Name, "NoEPGWorkaround"))  iepgSetup.NoEPGWorkaround = atoi(Value);

    else if (!strncasecmp(Name, "AutoUpdate", 10))  iepgSetup.SetupProviders[nIndex].autoupdate = atoi(Value);
    else if (!strncasecmp(Name, "UpdateTime", 10))  iepgSetup.SetupProviders[nIndex].updatetime = atoi(Value);
    else if (!strncasecmp(Name, "DaysInAdvance", 13))  iepgSetup.SetupProviders[nIndex].days = atoi(Value);
    else if (!strncasecmp(Name, "ProviderType", 12))
    {
        iepgSetup.SetupProviders[nIndex].ProviderType = atoi(Value);
        dsyslog("iepg: Setup [%s=%s] Set provider type[%d] = %d", Name, Value, nIndex, atoi(Value));
    }
    else if (!strncasecmp(Name, "Pin", 3))  strcpy(iepgSetup.SetupProviders[nIndex].Pin, Value);
    else if (!strncasecmp(Name, "BaseURL", 7))  strcpy(iepgSetup.SetupProviders[nIndex].baseurl, Value);
    else
        return false;
    return true;

}

cChannelSwitchStatusMonitor::cChannelSwitchStatusMonitor(void)
{
    sDir = strdup ( cPlugin::ConfigDirectory ( PLUGIN_NAME_I18N ));
}

cChannelSwitchStatusMonitor::~cChannelSwitchStatusMonitor()
{
    if (!sDir)
        free(sDir);
}

void cChannelSwitchStatusMonitor::ChannelSwitch(const cDevice *Device, int ChannelNumber)
{
    if (!StartupTime)
        return;

    if (ChannelNumber)
        dsyslog("iepg: Switch channel %d", ChannelNumber);
    else
        return;


    if ((time(NULL) - StartupTime) < WAIT_DEL_TIME)
    {
        return;
    }

    if (ChannelNumber >= MAX_VDR_CHANNELS)
        return;
    cChannel *chn = Channels.GetByNumber(ChannelNumber);

    if (!chn)
        return;
    if (!Reverse_ChannelList[ChannelNumber])
    {
        dsyslog("iepg: Channel %s not mapped to provider channel -> leave it untouched", chn->Name());
        return;
    }
    /*
        cCleanChannelData Metzger(NULL, chn->GetChannelID(), Reverse_ChannelList);
    //    Metzger.CleanNow();
        Metzger.Start();
    */

    if ( ThreadXml2vdr->isRunning() )
    {
        return;
    }

    if ( ThreadXml2vdr != NULL )
    {
        dsyslog ( "iepg: delete ThreadXmlEpg" );
        delete ThreadXml2vdr;
    }
    ThreadXml2vdr = new cThreadXmlEpg2vdr ( sDir, false, false, /* xCleanEpgOnly */ true ); // global object - supply copy of ConfigDirectory
    ThreadXml2vdr->Start();

}

void cRecordingsStatusMonitor::Recording(const cDevice *Device, const char *Name, const char *FileName, bool On, int ChannelNumber)
{
    if (!iepgSetup.save_rec_pic)	// copy of epg pictures not wanted -> return
        return;

    if (On)	// For safety and timing reasons copy image AFTER recording
        return;

    dsyslog("iepg: Status monitor: recording on  %s has ended", FileName);

    string sInfo = FileName;
    sInfo += "/info.vdr";

    FILE *fh = fopen(sInfo.c_str(), "r");	// open vdr.info in recording dir
    char sLine[512];
    int eventid;

    if (fh == NULL)
    {
        esyslog("iepg: File %s could not be opened. Aborting copy epg image to recording folder", sInfo.c_str());
        return;
    }
    bool bFound = false;

    while ( !feof ( fh ) && !bFound)  // search for event id ("E")
    {
        if (fgets(sLine, sizeof(sLine), fh) != NULL)
        {
            if (sLine[0] == 'E')
            {
                eventid = atoi(sLine + 2);
                bFound = true;
            }
        }
    }
    fclose(fh);

    if (!bFound)	// No event id found -> return
        return;

    char sImage[256];
    sprintf(sImage, "%s/%d.png", iepgSetup.sImageDirectory.c_str(), eventid);

    if (file_exists(sImage))
    {
        string sCommand = "cp ";
        sCommand += sImage;
        sCommand += " ";
        sCommand += FileName;
        sCommand += "/vdr.png";

        if (SystemExec(sCommand.c_str()))
            esyslog("iepg: Errors while copying epg image to recordings folder [%s]", sCommand.c_str());
    }
}

cStatusRecording::cStatusRecording(void)
{
    statusMonitor = NULL;
}

cStatusRecording::~cStatusRecording()
{
    delete statusMonitor;
}

bool cStatusRecording::Start(void)
{
    statusMonitor = new cRecordingsStatusMonitor;
    return true;
}

const char *cStatusRecording::Version(void)
{
    return 0;
}
const char *cStatusRecording::Description(void)
{
    return 0;
}


cStatusChannelSwitch::cStatusChannelSwitch(void)
{
    sDir = strdup ( cPlugin::ConfigDirectory ( PLUGIN_NAME_I18N ) );
    statusMonitor = NULL;
}

cStatusChannelSwitch::~cStatusChannelSwitch()
{
    delete statusMonitor;
    free(sDir);
}

bool cStatusChannelSwitch::Start(void)
{
    statusMonitor = new cChannelSwitchStatusMonitor;
    return true;
}

const char *cStatusChannelSwitch::Version(void)
{
    return 0;
}
const char *cStatusChannelSwitch::Description(void)
{
    return 0;
}

VDRPLUGINCREATOR (cPluginIEpg); // Don't touch this!
