/*
 * itools.h: TODO: Description
 *
 * See the README file for copyright information and how to reach the author.
 *
 * Copyright (C) 2008
*/

#ifndef __IEPG_TOOLS_H
#define __IEPG_TOOLS_H

#include <vdr/tools.h>
#include <iconv.h>

// int   charsetconv(const char *buffer, int buf_len, const char *str, int str_len, const char *from, const char *to);
char *striphtml(char *str);
void *myrealloc(void *ptr, size_t size);


// When handling strings that might contain UTF-8 characters, it may be necessary
// to process a "symbol" that consists of several actual character bytes. The
// following functions allow transparently accessing a "char *" string without
// having to worry about what character set is actually used.

int iUtf8CharLen(const char *s);
///< Returns the number of character bytes at the beginning of the given
///< string that form a UTF-8 symbol.
uint iUtf8CharGet(const char *s, int Length = 0);
///< Returns the UTF-8 symbol at the beginning of the given string.
///< Length can be given from a previous call to Utf8CharLen() to avoid calculating
///< it again. If no Length is given, Utf8CharLen() will be called.
int iUtf8CharSet(uint c, char *s = NULL);
///< Converts the given UTF-8 symbol to a sequence of character bytes and copies
///< them to the given string. Returns the number of bytes written. If no string
///< is given, only the number of bytes is returned and nothing is copied.
int iUtf8SymChars(const char *s, int Symbols);
///< Returns the number of character bytes at the beginning of the given
///< string that form at most the given number of UTF-8 Symbols.
int iUtf8StrLen(const char *s);
///< Returns the number of UTF-8 symbols formed by the given string of
///< character bytes.
char *iUtf8Strn0Cpy(char *Dest, const char *Src, int n);
///< Copies at most n character bytes from Src to Dst, making sure that the
///< resulting copy ends with a complete UTF-8 symbol. The copy is guaranteed
///< to be zero terminated.
///< Returns a pointer to Dest.
int iUtf8ToArray(const char *s, uint *a, int Size);
///< Converts the given character bytes (including the terminating 0) into an
///< array of UTF-8 symbols of the given Size. Returns the number of symbols
///< in the array (without the terminating 0).
int iUtf8FromArray(const uint *a, char *s, int Size, int Max = -1);
///< Converts the given array of UTF-8 symbols (including the terminating 0)
///< into a sequence of character bytes of at most Size length. Returns the
///< number of character bytes written (without the terminating 0).
///< If Max is given, only that many symbols will be converted.
///< The resulting string is always zero-terminated if Size is big enough.

// When allocating buffer space, make sure we reserve enough space to hold
// a string in UTF-8 representation:

#define iUtf8BufSize(s) ((s) * 4)

// The following macros automatically use the correct versions of the character
// class functions:

#define iUtf8to(conv, c) (cCharSetConv::SystemCharacterTable() ? to##conv(c) : tow##conv(c))
#define iUtf8is(ccls, c) (cCharSetConv::SystemCharacterTable() ? is##ccls(c) : isw##ccls(c))

class ciCharSetConv
{
private:
    iconv_t cd;
    char *result;
    size_t length;
    static char *systemCharacterTable;
public:
    ciCharSetConv(const char *FromCode = NULL, const char *ToCode = NULL);
    ///< Sets up a character set converter to convert from FromCode to ToCode.
    ///< If FromCode is NULL, the previously set systemCharacterTable is used.
    ///< If ToCode is NULL, "UTF-8" is used.
    ~ciCharSetConv();
    const char *Convert(const char *From, char *To = NULL, size_t ToLength = 0);
    ///< Converts the given Text from FromCode to ToCode (as set in the cosntructor).
    ///< If To is given, it is used to copy at most ToLength bytes of the result
    ///< (including the terminating 0) into that buffer. If To is not given,
    ///< the result is copied into a dynamically allocated buffer and is valid as
    ///< long as this object lives, or until the next call to Convert(). The
    ///< return value always points to the result if the conversion was successful
    ///< (even if a fixed size To buffer was given and the result didn't fit into
    ///< it). If the string could not be converted, the result points to the
    ///< original From string.
    static const char *SystemCharacterTable(void)
    {
        return systemCharacterTable;
    }
    static void SetSystemCharacterTable(const char *CharacterTable);
};


#endif // __IEPG_TOOLS_H

