/*
*  C Implementation: menu.c
*/
// Description: mapping of channels in the vdr to external sources
//
//
// Author: RealQuick <RealQuick@doubleQuick.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <vdr/status.h>
#include "i18n.h"
#include <vdr/menu.h>
#include "menu.h"
// #include <cstring>
#include <string>
#include "xmlepg2vdrthread.h"

#include <sstream>
#include <iomanip>
#include <fstream>

#include <vdr/osdbase.h>

#include "common.h"
#include "setup.h"

using namespace std;

#define MENU_TITLE tr("EPG via internet - main screen")

// ***** Globals  *************************************************************

extern cThreadXmlEpg2vdr *ThreadXml2vdr;
extern int *Reverse_ChannelList;

ciepgMenu::ciepgMenu(char *xDir) :cOsdMenu ( MENU_TITLE,  20, 16, 10, 5)
{
    mMenu = NULL;
    sDir = xDir;

    SetHelp ( tr ( "Start update" ), tr ( "Force update of EPG" ), NULL, tr ("Experts") );

    LogList.clear();
    logs l;

    l.description = tr ( "Last transmission" );
    l.logfile = iepgSetup.sDataDirectory.c_str(); l.logfile += "/thread.log";
    l.wdir = iepgSetup.sDataDirectory.c_str();

    struct stat tfstate;
    if ( !stat ( l.logfile.c_str(), &tfstate ) )
    {
        struct tm *ttm = localtime ( &tfstate.st_mtime );
        char *dt = NULL;
        asprintf ( &dt, "%02d.%02d.%04d %02d:%02d:%02d", ttm->tm_mday, ttm->tm_mon+1, ttm->tm_year + 1900, ttm->tm_hour, ttm->tm_min, ttm->tm_sec );
        l.date = dt;
        free ( dt );
    }

    LogList.push_back ( l );


    char *file;
    file = ( char * ) malloc ( 512 );

    DIR *dir = NULL;

    // search for logfiles
    string checkdir = iepgSetup.sDataDirectory; checkdir += "/1";
    if ( ( dir = opendir ( checkdir.c_str() ) ) != NULL )
    {
        struct dirent *dirent = NULL;
        struct stat fstate;

        // loop through directories
        while ( ( dirent = readdir ( dir ) ) != NULL )
        {
            string file = checkdir; file += "/";
            file += dirent->d_name;
            if ( !stat ( file.c_str(), &fstate ) )
            {
                // for each directory look for logfile (/import.log)
                if ( S_ISDIR ( fstate.st_mode ) )
                {
                    l.wdir = file; // save directory name
                    file += "/";
                    file += LOGFILE_IMPORT;
                    if ( !stat ( file.c_str(), &fstate ) )
                    {
                        struct tm *tm = localtime ( &fstate.st_mtime );
                        char *d = NULL;
                        asprintf ( &d, "%02d.%02d.%04d %02d:%02d:%02d", tm->tm_mday, tm->tm_mon+1, tm->tm_year + 1900, tm->tm_hour, tm->tm_min, tm->tm_sec );
                        l.date = d;
                        setup_utf8 ( &l.date );  // march has come and now month contains umlauts
                        free ( d );
                        l.description = tr ( "epg data" );
                        l.description += " ";
                        l.description += dirent->d_name;
                        l.logfile = file;

                        LogList.push_back ( l );
                    }
                }
            }
        }
	closedir(dir);
    }

    for ( unsigned int nIndex=0;nIndex<LogList.size();nIndex++ )
    {
        stringstream sout;

        FILE *fh;
        string sFile = LogList[nIndex].wdir;
        sFile += "/";
        sFile += LOGFILE_STATUS;

        char buff[64];
        fh = fopen ( sFile.c_str(), "r" );
        if ( fh )
        {
            fgets ( buff, sizeof ( buff ), fh );
            fclose ( fh );
            buff[strlen(buff) - 1] = '\0';	// remove EOF
        }
        else
            sprintf ( buff, "                %s", tr ( "unknown" ) );

	string sDate = LogList[nIndex].date;
        setup_utf8 ( &sDate );  // march has come and now month contains umlauts --> convert 

        sout << LogList[nIndex].description << "\t" << sDate << "\t" << buff+16;

        Add ( new cOsdItem ( sout.str().c_str() ), 1 );
    }

    SetCurrent ( NULL );
    Display();
}

ciepgMenu::~ciepgMenu()
{
    if(!mMenu)
	delete mMenu;
}

void ciepgMenu::Display ( void )
{
    cOsdMenu::Display();
// ?????  DisplayMenu()->SetText(score, false);
//   cStatus::MsgOsdTextItem(score);
}

eOSState ciepgMenu::ProcessKey ( eKeys Key )
{
    string sMenuText;

    eOSState state = cOsdMenu::ProcessKey ( Key );
    if ( state == osUnknown )
    {
        switch ( Key )
        {
        case  kOk:
            FILE *fh;

            fh = fopen ( LogList[Current() ].logfile.c_str(), "r" );
            if ( fh )
            {
                fseek ( fh, 0, SEEK_END );
                int nSize = ftell ( fh );
                fseek ( fh, 0, SEEK_SET );

                char *buffer = ( char * ) malloc ( nSize + 1 );

                fread ( buffer, nSize, 1, fh );
                fclose ( fh );

                buffer[nSize] = '\0';

                state =  AddSubMenu ( new cMenuText ( tr ( "Show logfile" ), buffer ) );

                free ( buffer );

                return state;
            }
            break;

        case kRed:
            if ( ThreadXml2vdr->isRunning() )
            {
                Skins.Message ( mtInfo, tr ( "EPG import is currently running" ), 1 );
                break;
            }
	    ThreadXml2vdr->SetQuiet(false);
            ThreadXml2vdr->Start();
            break;

        case kGreen:
            if ( ThreadXml2vdr->isRunning() )
            {
                Skins.Message ( mtInfo, tr ( "EPG import is currently running" ), 1 );
                break;
            }

            if ( ThreadXml2vdr != NULL )
            {
                dsyslog ( "iepg: delete ThreadXmlEpg" );
                delete ThreadXml2vdr;
            }

            ThreadXml2vdr = new cThreadXmlEpg2vdr ( sDir, true /* Force */, false /* Default */, false /* Default */, false /* not to be quiet*/ ); // global object - supply copy of ConfigDirectory
            ThreadXml2vdr->Start();
            break;
        case kYellow:
        case kBlue:
	    sMenuText = "\n";
	    sMenuText += tr("In this submenu you have the option to do some administrative tasks.");
	    sMenuText += "\n\n";
	    sMenuText += tr("Cleaning epg data will trigger the plugin to search for duplicate and unwanted epg entries ");
	    sMenuText += tr("and eliminates them by either moving or removing.");
	    sMenuText += "\n";
	    sMenuText += tr("Note: It is not guaranteed, that after performing the cleaning twice, the second pass detects no more duplicate entries.");
	    sMenuText += tr("Especially, if the schedules differs very much due a faulty channelmapping, iepg will not be able to eliminate all unwanted entries.");
	    sMenuText += "\n\n";
	    sMenuText += tr("Reverting the data will remove all previous downloaded and converted files.");
	    sMenuText += tr("Consider using this option in case of inconsistences due a faulty installation or downlad erros / processing.");
	    sMenuText += "\n";
	    AddSubMenu(new ciepgExpertMenu(tr("iepg - experts menu"), sMenuText.c_str()));
            break;
//        case kHelp: 
//	    showHelp (); 
//	    state = osContinue;
//	    break;
        default:
            break;
        }
        state = osContinue;
    }
    return state;
}


void ciepgMenu::showHelp()
{
/*
    if (!HelpMenus.Load())
        return;
    cHelpSection *hs = HelpMenus.GetSectionByTitle(MENU_TITLE);
    mMenu = new cMenuHelp(hs, "TEST TEST TEST TEST");
    mMenu->Display();
*/
}

ciepgExpertMenu::ciepgExpertMenu(const char *xTitle, const char *xText)
:cMenuText(xTitle, xText)
{
	SetHelp(tr("Revert All"), tr("Clean EPG"));
}

eOSState ciepgExpertMenu::ProcessKey(eKeys Key)
{
    int nCount;
    string sImageDir = iepgSetup.sImageDirectory;
    string sIepgDir = iepgSetup.sDataDirectory; sIepgDir += "/";
    std::stringstream sOut;

  eOSState state = cOsdMenu::ProcessKey(Key);
  if (state == osUnknown) {
    switch  (Key) 
    {
      case  kOk:
            state = osContinue;
            break;
      case  kGreen:
	    if(Reverse_ChannelList)
	    {
        	cCleanChannelData Metzger(NULL, tChannelID(), Reverse_ChannelList);
        	nCount = Metzger.CleanNow(false);

		sOut << tr ( "EPG data has been successfully cleaned!") << "(" << nCount << " " << tr("entries moved/removed)");
	        Skins.Message( mtInfo, sOut.str().c_str());
	    } else
	        Skins.Message( mtError, (char *)tr ( "Cleaning of EPG data could not be done!" ));	

            state = osContinue;
	    break;
      case  kBlue:
            state = osContinue;
	    break;
      case  kRed:
	    if (ThreadXml2vdr)
	    {
                if (ThreadXml2vdr->isRunning())
		{
                    Skins.Message( mtError, tr("EPG import is currently running"));
		    return osContinue;
		}
	    }

    	    // Cleanup raw Pictures
    	    nCount = Cleanup ( ( char * ) sImageDir.c_str(), ".raw", 365 );
    	    // Cleanup PNG Pictures
    	    nCount = Cleanup ( ( char * ) sImageDir.c_str(), ".png", 365 );
    	    // Cleanup Directories

            nCount = 0;
            for(unsigned int i=1;i<iepgSetup.SetupProviders.size();i++)
            {
	       string sDirToClean = sIepgDir;
               sDirToClean += itoa(i);
    	       nCount += Cleanup (sDirToClean.c_str(), ".", 365 );
            }
	    Skins.Message( mtInfo, (char *)tr ( "All previously downloaded data has been removed!" ));
            state = osContinue;
	    break;
      case  kYellow:
            state = osContinue;
	    break;
      default:
            state = osContinue;
            break;
    }
  }
  return state;
}


