/*
*  C Implementation: misc.c
*/
// Description: Some miscellanous functions
//
//
// Author: RealQuick <RealQuick@doubleQuick.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include <time.h>
#include <stdio.h>
#include <string.h>  // strdup()
#include <string>

#include <vector>

#include <vdr/channels.h>       // for Channels

#include "itools.h"
#include "setup.h"

// #include "misc.h"

#include <vdr/osdbase.h>
#include "common.h"

#include <utime.h>  // for setting creation date of files
#include <sys/time.h>

#include "xmlepg2vdrthread.h"

using namespace std;

/********* str2time_t *************************************/
time_t
str2time_t ( const char *parmTime, const char *parmDate = NULL, int jumpDays = 0 )
{
    /*
     *   parmTime is "1130" not "11:30"
     *   parmDate is "dd/mm/yyyy" for eg. "17/11/2007"
     *
     */
    time_t t1;
    char trailer;
    t1 = time ( NULL );
    struct tm when;               /* = *localtime(&t1); */

    if ( parmDate != NULL )
    {
        sscanf ( parmDate, " %d/%d/%d%c", & ( when.tm_mday ), & ( when.tm_mon ), & ( when.tm_year ), &trailer );
        if ( when.tm_year >= 1900 )
            when.tm_year -= 1900;
        when.tm_mon--;
        when.tm_isdst = 0;
        when.tm_yday = 0;
    }
    else
    {
        when = *localtime ( &t1 );
    }
    sscanf ( parmTime, " %d%c", & ( when.tm_hour ), &trailer );
    when.tm_sec = 0;
    when.tm_min = when.tm_hour % 100;
    when.tm_hour /= 100;
//    showTM(when);

    return mktime ( &when ) + ( time_t ) ( 24 * 60 * 60 * jumpDays );
}

/********* dtstr2time_t *************************************/
time_t
dtstr2time_t ( const char *parmTime )
{
    /*
     *   Date is "yyyy-mm-dd hh:mm:ss"
     *
     */
    time_t t1;
    t1 = time ( NULL );
    struct tm when;               /* = *localtime(&t1); */

    if ( parmTime != NULL )
    {
        sscanf ( parmTime, "%04d-%02d-%02d %02d:%02d:%02d", & ( when.tm_year ), & ( when.tm_mon ), & ( when.tm_mday ), & ( when.tm_hour ), & ( when.tm_min ), & ( when.tm_sec ) );

// dsyslog("iepg: [%s] -> %02d.%02d.%04d %02d:%02d:%02d", parmTime, when.tm_mday, when.tm_mon, when.tm_year, when.tm_hour,when.tm_min,when.tm_sec);
    }
    else
    {
        when = *localtime ( &t1 );
    }

    when.tm_year -= 1900;  // only 2 digits for year
    when.tm_mon--;  // Month is zero based

    return mktime ( &when );
}

int knuth_hash(unsigned char *str, int base)
{
    unsigned long hash;
    int len = strlen((char *)str);

    for (hash=len; len--;)
    {
        hash = ((hash<<5)^(hash>>27))^*str++;
    }
    return (int)(hash % base);     // % prime
}

int GetHashIndex(cProviderChannelList *ChannelList, const char *ProviderChannel)
{
    int hash;

    hash = MAX_PROVIDER_CHANNEL + knuth_hash((unsigned char*)ProviderChannel, PROVIDER_HASHSIZE);

    if (ChannelList[hash].sHashString == ProviderChannel)
    {
        return hash;	// entry was found
    }

    // search sequential in collision table
    for (int x=MAX_PROVIDER_CHANNEL + PROVIDER_HASHSIZE;ChannelList[hash].sHashString.size() && x < (MAX_PROVIDER_CHANNEL + PROVIDER_HASHSIZE + COLLISION_TABSIZE);x++)
    {
        if (ChannelList[x].sHashString == ProviderChannel)
        {
            return x;	// entry was found
        }
    }

    return 0;	// Not found
}

// setup array of ChannelIds  array[ProviderChannel] --> ChannelId vdr  (much memory consuming but fast)
bool SetChannelData ( cProviderChannelList *ChannelList, const char *channelName, int *ProviderChannel, const char *ProviderChannelString, int nProvider )
{
    int hash;
    string sHashStringUsed;
    bool bChannelFound;

    bChannelFound = false;

    if (*ProviderChannel != 0)  	// Provider channel provided -> use as index
    {
        char *stmp;
        asprintf(&stmp, "%d", *ProviderChannel);
        hash = MAX_PROVIDER_CHANNEL + knuth_hash((unsigned char*)stmp, PROVIDER_HASHSIZE);

        sHashStringUsed = stmp;

        free(stmp);
    }
    else
    {
        hash = MAX_PROVIDER_CHANNEL + knuth_hash((unsigned char *)ProviderChannelString, PROVIDER_HASHSIZE);

        sHashStringUsed = ProviderChannelString;
    }

    {
// position is free -> take it
        if (!ChannelList[hash].sHashString.size())
        {
            // ProviderChannelString is set -> Use value as Channel-Name
            if (ProviderChannelString)
                ChannelList[hash].ChannelName = ProviderChannelString;
            else
                ChannelList[hash].ChannelName = channelName;

            ChannelList[hash].sHashString = sHashStringUsed;
            ChannelList[hash].Provider = nProvider;
            *ProviderChannel = hash;
            bChannelFound = true;
//            dsyslog("iepg: hash_value for %s = %d OK", ProviderChannelString, hash);
        }
        else   // search free position in collision table
        {
            // Is entry already in table?
            if (ChannelList[hash].sHashString == sHashStringUsed)
            {
                *ProviderChannel = hash;
                bChannelFound = true;
//	    	return true;  // Entry is already in table
            }

//            dsyslog("iepg: hash_value for %s = %d not free -> serach collision table", ProviderChannelString, hash);
            for (int x=MAX_PROVIDER_CHANNEL + PROVIDER_HASHSIZE; !bChannelFound & x < (MAX_PROVIDER_CHANNEL + PROVIDER_HASHSIZE + COLLISION_TABSIZE);x++)
            {
                if (ChannelList[x].sHashString.size())
                {
                    if (ChannelList[x].sHashString == sHashStringUsed)
                    {
                        *ProviderChannel = hash = x;
                        bChannelFound = true;
//		    	return true;  // Entry is already in Collision table
                    }
                }
                else
                {
                    // free position reached -> use this free slot
                    if (ProviderChannelString)
                        ChannelList[x].ChannelName = ProviderChannelString;
                    else
                        ChannelList[x].ChannelName = channelName;
                    *ProviderChannel = hash = x;
                    ChannelList[x].sHashString = sHashStringUsed;
                    ChannelList[x].Provider = nProvider;
                    bChannelFound = true;
                    dsyslog("iepg: Channel [%s] Index for %s = %d in collision table", channelName, ProviderChannelString, x);
                }
            }
            if (!bChannelFound)
            {
                esyslog("iepg: Index for %s no empty slots in collision table", ProviderChannelString);
                return false;
            }

        }
        if (channelName != NULL && bChannelFound)
        {
            for ( cChannel * c = Channels.First (); c; c = Channels.Next ( c ) )
            {
                if ( strcasecmp ( c->Name(), channelName ) == 0 )
                {
                    tChannelID id = c->GetChannelID ();
                    ChannelList[hash].ChannelId = id;
                    ChannelList[hash].Provider = nProvider;
                    return true;
                }
            }
            return true;
        }
        else
        {
            dsyslog("iepg: SetChanneldata: No Channel (%s) provided for search in channels.conf", channelName);
            return true;
        }
    }
    return false;
}


/*******CRC16 ****************************************/

#define POLY 0xA001             // CRC16

unsigned int
crc16 ( unsigned int crc, unsigned char const *p, int len )
{
    while ( len-- )
    {
        crc ^= *p++;
        for ( int i = 0; i < 8; i++ )
            crc = ( crc & 1 ) ? ( crc >> 1 ) ^ POLY : ( crc >> 1 );
    }
    return crc & 0xFFFF;
}

char *get_dir_per_day ( char *path, int day )
{
    time_t now;
    struct tm *tm;
    char sNow[256];

    now = time ( NULL )  + ( time_t ) ( day * 24 * 60 * 60 );
    tm = localtime ( &now );
    sprintf ( sNow, "%s/%04d%02d%02d", path, tm->tm_year + 1900,tm->tm_mon+1,tm->tm_mday );
    return ( strdup ( sNow ) );
}

char *get_xml_from_zip ( char *path )
{
    DIR *dp;
    struct dirent *ep;

    char *Return;

    dp = opendir ( path );
    if ( dp != NULL )
    {
        while ( ( ep = readdir ( dp ) ) != NULL )
        {
            if ( ep->d_type != DT_DIR )
            {
                if ( strstr ( ep->d_name, ".xml" ) != NULL && strstr ( ep->d_name, "processed" ) == NULL ) // search for XML fild, not processed
                {
                    Return = strdup ( ep->d_name );
                    closedir ( dp );
                    return Return;
                }
            }
        }
        closedir ( dp );

        {
            esyslog ( "iepg: xml file not found in %s", path );
            return NULL;
        }
    }
    else
    {
        esyslog ( "iepg: Couldn't open the directory %s", path );
        return NULL;
    }
}

bool file_exists ( char *path )
{
    struct stat results;
    if ( stat ( path, &results ) == 0 )
        return true;
    else
        return false;
}

// save log to file
void do_log ( const char *path, const char *logfile, const char *buf, bool newfile )
{
    FILE *f;

    time_t t=time ( NULL );
    struct tm *tm=localtime ( &t );
    char dtstr[20];

    char lf[512];

    sprintf ( lf, "%s/%s", path, logfile );
    f=fopen ( lf, newfile ? "w" : "a" );
    if ( f )
    {
        strftime ( dtstr, sizeof ( dtstr ), "%b %e %T", tm );
        fprintf ( f,"%s %s\n",dtstr, buf );
        fclose ( f );
    }
}

void touch_file ( char *path, time_t time_to_set )
{
    struct stat fstate;
    time_t now;

    now = time ( NULL );

    if ( stat ( path, &fstate ) )
    {
        esyslog ( "iepg: could not examine file %s ", path );
        return;
    }

    struct tm *tm;

    // date is newer then actual creation date -> increase date to protect from being removed
    if ( time_to_set > fstate.st_ctime )
    {

        struct utimbuf buf;
        buf.modtime = time_to_set;
        buf.actime = now;

        dsyslog ( "iepg: touch file: %s", path );
        if ( utime ( path, &buf ) )
            esyslog ( "iepg: touching of file %s failed with status %d, %s", path, errno, strerror ( errno ) );


        /*
        struct timeval timevals[2];

        timevals[0].tv_sec=now;
        timevals[0].tv_usec=0;

        timevals[1].tv_sec=now;
        timevals[1].tv_usec=0;

         if(lutimes(path, timevals))
                     esyslog("iepg: touching of file %s failed with status %d, %s", path, errno, strerror(errno));
        */


    }
    else
    {
        struct tm *tm_file;
        tm = localtime ( &now );
        tm_file = localtime ( &fstate.st_ctime );
        dsyslog ( "iepg: file not touched %s (%04d%02d%02d%02d%02d.%02d) <= (%04d%02d%02d%02d%02d.%02d)", path, tm->tm_year + 1900, tm->tm_mon+1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec, tm_file->tm_year + 1900, tm_file->tm_mon+1, tm_file->tm_mday, tm_file->tm_hour, tm->tm_min, tm_file->tm_sec );
    }
}

void setup_utf8 ( string *str )
{
    char *tmp;
    char *out;


    int nLen = strlen ( str->c_str() ) + 1;

    tmp = ( char * ) malloc ( nLen+1 );
    if ( tmp == NULL )
    {
        esyslog ( "iepg: malloc for tmp (%d) failed", nLen );
        return;
    }
    out = ( char * ) malloc ( 2 * nLen+1 );
    if ( out == NULL )
    {
        esyslog ( "iepg: malloc for out (%d) failed", 2 * nLen );
        return;
    }

    memset ( tmp, 0, nLen );
    memset ( out, 0, nLen * 2);

    /*
    #if defined(APIVERSNUM) && APIVERSNUM < 10412
        charsetconv ( tmp, nLen, ( char * ) str->c_str(), strlen ( ( char * ) str->c_str() ), "UTF8", I18nCharSets() [Setup.OSDLanguage] );
    #else
        const char* codeset =  cCharSetConv::SystemCharacterTable();
        charsetconv ( tmp, nLen, str, strlen ( ( char * ) str->c_str() ), "UTF8", codeset?codeset:"UTF-8" );
    #endif
    */

    strn0cpy ( tmp, str->c_str(), nLen );


    compactspace ( tmp );
    striphtml ( tmp );

#if RBLITE
    strn0cpy ( out, tmp, nLen );
#else
//    iUtf8Strn0Cpy(out, tmp, nLen);

    ciCharSetConv conv = ciCharSetConv("ISO-8859-1", "UTF-8");
    conv.Convert(tmp, out, nLen*2);

//dsyslog("iepg: AVG utf8-conv %s -> %s", tmp, out);
#endif

    *str = "";
    *str += out;

    free ( tmp );
    free ( out );
}

// Get description of the genre id
const char *get_genre ( string genre_id )
{
    switch ( atoi ( genre_id.c_str() ) )
    {
    case 101:
        return tr ( "Movie" );
    case 102:
        return tr ( "Adventure" );
    case 103:
        return tr ( "Action" );
    case 104:
        return tr ( "Documentation" );
    case 105:
        return tr ( "Drama" );
    case 106:
        return tr ( "Erotic" );
    case 108:
        return tr ( "Fantasy" );
    case 109:
        return tr ( "Film with reg. bg" );
    case 110:
        return tr ( "Humor" );
    case 112:
        return tr ( "Thriller/Crime" );
    case 113:
        return tr ( "Culture" );
    case 114:
        return tr ( "Filmlet" );
    case 115:
        return tr ( "Music" );
    case 116:
        return tr ( "Mystery+Horror" );
    case 117:
        return tr ( "Romance/Love" );
    case 119:
        return tr ( "Science Fiction" );
    case 121:
        return tr ( "Thriller" );
    case 122:
        return tr ( "Western" );
    case 123:
        return tr ( "Animation" );
    case 201:
        return tr ( "Serial" );
    case 202:
        return tr ( "Adventure" );
    case 203:
        return tr ( "Action" );
    case 205:
        return tr ( "Drama" );
    case 206:
        return tr ( "Erotic" );
    case 207:
        return tr ( "Family" );
    case 208:
        return tr ( "Fantasy" );
    case 210:
        return tr ( "Humor" );
    case 211:
        return tr ( "Hospital" );
    case 212:
        return tr ( "Thriller/Crime" );
    case 214:
        return tr ( "Youth" );
    case 216:
        return tr ( "Mystery+Horror" );
    case 218:
        return tr ( "Reality" );
    case 219:
        return tr ( "Science Fiction" );
    case 220:
        return tr ( "Soap" );
    case 221:
        return tr ( "Thriller" );
    case 222:
        return tr ( "Western" );
    case 223:
        return tr ( "Animation" );
    case 301:
        return tr ( "Sports" );
    case 331:
        return tr ( "Boxing" );
    case 332:
        return tr ( "Icehockey" );
    case 334:
        return tr ( "Soccer" );
    case 335:
        return tr ( "Olympia" );
    case 336:
        return tr ( "Golf" );
    case 337:
        return tr ( "Gymnastic" );
    case 338:
        return tr ( "Handball" );
    case 339:
        return tr ( "Motorsport" );
    case 340:
        return tr ( "Cycling" );
    case 341:
        return tr ( "Tennis" );
    case 342:
        return tr ( "Water sports" );
    case 343:
        return tr ( "Winter sports" );
    case 344:
        return tr ( "US-Sport" );
    case 345:
        return tr ( "Athletics" );
    case 346:
        return tr ( "Volleyball" );
    case 347:
        return tr ( "Extremsport" );
    case 348:
        return tr ( "Reportage" );
    case 401:
        return tr ( "Show" );
    case 406:
        return tr ( "Erotic" );
    case 418:
        return tr ( "Reality" );
    case 450:
        return tr ( "Comedy" );
    case 451:
        return tr ( "Familiy show" );
    case 452:
        return tr ( "Gameshows" );
    case 453:
        return tr ( "Talkshows" );
    case 454:
        return tr ( "Courtroom drama" );
    case 455:
        return tr ( "Homeshopping" );
    case 456:
        return tr ( "Cooking show" );
    case 457:
        return tr ( "Home working" );
    case 501:
        return tr ( "Information" );
    case 560:
        return tr ( "History" );
    case 561:
        return tr ( "Magazin" );
    case 564:
        return tr ( "Health" );
    case 565:
        return tr ( "Motor+traffic" );
    case 566:
        return tr ( "News" );
    case 567:
        return tr ( "Nature" );
    case 568:
        return tr ( "Politics" );
    case 569:
        return tr ( "Guidings" );
    case 570:
        return tr ( "Travel" );
    case 571:
        return tr ( "Economy" );
    case 572:
        return tr ( "Knowledge" );
    case 573:
        return tr ( "Documentation" );
    case 601:
        return tr ( "Culture" );
    case 680:
        return tr ( "Jazz" );
    case 681:
        return tr ( "Classical music" );
    case 682:
        return tr ( "Musical" );
    case 683:
        return tr ( "Rock" );
    case 684:
        return tr ( "Folk music" );
    case 685:
        return tr ( "Alternative" );
    case 686:
        return tr ( "Pop" );
    case 687:
        return tr ( "Clips" );
    case 688:
        return tr ( "Show" );
    case 689:
        return tr ( "Interview" );
    case 690:
        return tr ( "Theater" );
    case 691:
        return tr ( "Cinema" );
    case 692:
        return tr ( "Culture" );
    case 701:
        return tr ( "Children" );
    case 790:
        return tr ( "Children's film" );
    case 791:
        return tr ( "Children's news" );
    case 792:
        return tr ( "Children's serials" );
    case 793:
        return tr ( "Children's shows" );
    case 795:
        return tr ( "Animation" );
    case 796:
        return tr ( "Anime" );
    default:
        return "";
    }
}


int Cleanup ( const char *path, const char *ext, int nDaysBack )
{
    cURLFile tmp_obj;

    char *file;
    file = ( char * ) malloc ( 512 );

    DIR *dir = NULL;
    int nCount = 0;

    if ( ( dir = opendir ( path ) ) != NULL )
    {
        struct dirent *dirent = NULL;
        struct stat fstate;
        time_t today = time ( NULL ) + (nDaysBack * 24 * 60 * 60);  // clean up before this date
        struct tm *tm = localtime ( &today );
        tm->tm_hour = 0;
        tm->tm_min = 0;
        tm->tm_sec = 0;
        today = mktime ( tm );

	dsyslog("iepg: cleanup files before %s in path %s (%s)", asctime(tm), path, ext);

	std::vector < string > DirectoriesToDelete;
	DirectoriesToDelete.clear();
        while ( ( dirent = readdir ( dir ) ) != NULL )
        {
            char *filename = NULL;

            if ( asprintf ( &filename, "%s/%s", path, dirent->d_name ) == -1 )
                esyslog ( "iepg: failed to allocate string" );

            if ( filename )
            {
                realpath ( filename, file );
                if ( !stat ( file, &fstate ) )
                {
// dsyslog("iepg: check file: %s", dirent->d_name);
                    // ext = '.' and is directory
                    if (S_ISDIR ( fstate.st_mode ) && *ext == '.')
                    {
// dsyslog("iepg: check file: %s -> IS DIR", dirent->d_name);
                        // download folder name YYYYMMDD
                        if (atoi(dirent->d_name) > 20090000 && atoi(dirent->d_name) < 29990000)
                        {
                            int nToOld = (tm->tm_year + 1900) * 100 * 100 + (tm->tm_mon+1) * 100 + tm->tm_mday;
                            if (atoi(dirent->d_name) < nToOld)
                            {
                                char sRemoveDir[512];
                                sprintf ( sRemoveDir, "rm -rf %s/%s", path, dirent->d_name);
				DirectoriesToDelete.push_back(sRemoveDir);
                            }
                        }
                    }

                    if ( !strncmp ( dirent->d_name + strlen ( dirent->d_name ) -
                                    strlen ( ext ), ext, strlen ( ext ) ) &&
                            S_ISREG ( fstate.st_mode ) && ( fstate.st_mtime < today ) )
                    {
                        if ( !unlink ( filename ) )
                        {
                            isyslog ( "Removed outdated file %s", filename );
                            nCount++;
                        }
                        else
                        {
                            esyslog ( "iepg: Can't remove file %s", filename );
                        }
                    }
                }
                else
                {
                    esyslog ( "iepg: Can't get informations for file %s", filename );
                }

                free ( filename );
                filename = NULL;
            }
            else
            {
                esyslog ( "iepg: Can't allocate enough memory for filename" );
            }
        }
        closedir ( dir );

	for(unsigned int x=0;x< DirectoriesToDelete.size();x++)
	{
            nCount++;
            SystemExec ( DirectoriesToDelete[x].c_str() );
dsyslog("iepg: remove dir: %s", DirectoriesToDelete[x].c_str());
	}
    }
    else
    {
        esyslog ( "iepg: Can't open directory %s", path );
    }

    free ( file );

    return nCount;
}

bool ReadUrlList ( cThreadXmlEpg2vdr *p, std::vector < cURLFile > *Urls, const char *sConfigDir, int nProvider, cProviderChannelList *ChannelList, bool Download, int WorkingDay, bool *bLoop )
{
    bool ChannelList_Complete;

    char url_str[1024], line[1024], line_org[1024];
    cURLFile tmp_obj;

    DIR *tmpdir;
    char sDir[256];

    char ch, tmp_EpgFileName[256], channelid[10];
    char channelName_str[128];
    char urlListFileName[512];

    // Dont construct Channel-List twice
    if (Urls->size())
        ChannelList_Complete = true;
    else
        ChannelList_Complete = false;

    sprintf ( urlListFileName, "%s/%d/%s", sConfigDir, nProvider, URLLISTFILENAME );
    FILE *urlFile;

    /********* open URL file ***************/
    urlFile = fopen ( urlListFileName, "r" );
    if ( urlFile == NULL )
    {
        //TODO error msg
        esyslog ( "iepg: error opening %s. Aborting", urlListFileName );
        return false;
    }

    dsyslog ( "iepg: opened URLLISTFILE %s for reading.", urlListFileName );

    int i = 0;
    /*
     * first get all the XML(EPG) files
     */

    if ( Download )
        dsyslog ( "iepg: downloading URLs from %s", urlListFileName );
    while ( /* Running () && */ !feof ( urlFile ) )
    {
        /* parsing URL file */
        if (fscanf ( urlFile, "%[^\n] ", line_org ) > 0)
        {

            strcpy ( line, line_org );

// Replacement of PIN and Workinf day is now made at end of function
            /*
                    // supply url-string with pin and / or day of desired epg - if configured
                    if ( strstr ( line, "%s" ) != NULL && strstr ( line, "%d" ) != NULL )
                        sprintf ( line, line_org, iepgSetup.SetupProviders[nProvider].Pin, WorkingDay );
                    else if ( strstr ( line, "%s" ) != NULL )
                        sprintf ( line, line_org, iepgSetup.SetupProviders[nProvider].Pin );
                    else if ( strstr ( line, "%d" ) != NULL )
                        sprintf ( line, line_org, WorkingDay );
            */
            url_str[0] = '\0';

            sscanf ( line, "%[^;] %c %[^\n]", channelName_str, &ch, url_str ); // 'ch' removes any trailing characters like ';'


            if (!ChannelList_Complete)
            {
                // TODO handle error in (parsing) URL.list
                if ( strstr ( channelName_str, "|" ) != NULL )
                {
                    char sDummy[40];
                    sscanf ( channelName_str, "%[^|] %s", sDummy, channelid );
                    tmp_obj.ChannelName = sDummy;
                    tmp_obj.ChannelId = channelid+1;
                    tmp_obj.ChannelURL = "";
                }
                else
                {
                    tmp_obj.ChannelName = channelName_str;

                    string tmp = url_str;
                    int nPos = tmp.rfind('/', -1);

                    tmp_obj.ChannelId = "0";  // No channeld id -> calculate hash
                    if (nPos > 0)
                    {
                        tmp_obj.ChannelURL = tmp.substr(nPos+1); // value in SetChannelData
                    }
                    else
                        esyslog("iepg:  / is missing in URL (%s)", url_str);
                }

                // Stoe providertype from setup
                tmp_obj.ProviderType = iepgSetup.SetupProviders[nProvider].ProviderType;

                dsyslog("iepg: channelName_str=%s, id=%s", channelName_str, tmp_obj.ChannelId.c_str());
                // Update array of Channel ids
                {
                    int cID = atoi ( tmp_obj.ChannelId.c_str() );
                    if ( !SetChannelData ( ChannelList, tmp_obj.ChannelName.c_str(), &cID, (char *)tmp_obj.ChannelURL.c_str(), nProvider ))
                    {
                        esyslog ( "iepg Update of channels for Channel %s and provider channelid %d failed", tmp_obj.ChannelName.c_str(), cID );
                        tmp_obj.isError = true;
                    }
                    else
                    {
                        dsyslog ( "iepg Update of channels for Channel %s and provider channelid %d successful", tmp_obj.ChannelName.c_str(), cID );

                        // If Channelid is not set -> Use (returned) hash value from SetChannelData
// ALWAYS USE HASH VALUE AS INDEX
//                    if (tmp_obj.ChannelId == "0")
                        tmp_obj.ChannelId = itoa(cID);

                    }
                }

                tmp_obj.ChannelURL = url_str;
            }
            else
// restore basic setting from previous list
// EpgFileName need to be modified (contains Day offset)
            {
                tmp_obj.ChannelName = Urls->at(i).ChannelName;
                tmp_obj.ChannelId = Urls->at(i).ChannelId;
                tmp_obj.ChannelURL = Urls->at(i).ChannelURL;
                tmp_obj.ProviderType = Urls->at(i).ProviderType;
                tmp_obj.isError = false;
            }
            sprintf ( sDir, "%s/%d", iepgSetup.sDataDirectory.c_str(), nProvider );

            // provider type 0 means pure xml file, 1 means all data is delivered in one zip file
            sprintf ( tmp_EpgFileName, "%s/epg%d_%d.%s", sDir, i, WorkingDay, iepgSetup.SetupProviders[nProvider].ProviderType == 0 ? "xml" : "zip" );

            tmp_obj.ProviderType = iepgSetup.SetupProviders[nProvider].ProviderType;

            // If flag Download is false -> Skip all further action
            if ( Download )
            {
                if ( ( tmpdir=opendir ( sDir ) ) == NULL )
                {
                    isyslog ( "Creating %s\n", sDir );
                    if ( mkdir ( sDir, ACCESSPERMS ) )
                    {
                        esyslog ( "Creation of %s failed.\n", sDir );
                        // isAlreadyRunning = false;

		        fclose ( urlFile );

                        return false;
                    }
                }
                else
                {
                    closedir ( tmpdir );
                }
                // xml file for provider type 1 is packed in an zip archive
                if ( iepgSetup.SetupProviders[nProvider].ProviderType == 0 )
                {
                    tmp_obj.EpgFileName = tmp_EpgFileName;
                }
                else
                {
                    char *new_dir = get_dir_per_day ( sDir, WorkingDay );
                    if ( ( tmpdir=opendir ( new_dir ) ) == NULL )
                    {
                        isyslog ( "Creating %s\n", new_dir );
                        if ( mkdir ( new_dir, ACCESSPERMS ) )
                        {
                            esyslog ( "Creation of %s failed.\n", new_dir );
//                    isAlreadyRunning = false;
                            free ( new_dir );

		            fclose ( urlFile );

                            return false;
                        }
                    }
                    else
                    {
                        closedir ( tmpdir );
                    }

                    free ( new_dir );
                }
                // Create directory for images if not already exist
                if ( ( tmpdir=opendir ( iepgSetup.sImageDirectory.c_str() ) ) == NULL )
                {
                    isyslog ( "Creating %s\n", iepgSetup.sImageDirectory.c_str() );
                    if ( mkdir ( iepgSetup.sImageDirectory.c_str(), ACCESSPERMS ) )
                    {
                        esyslog ( "Creation of %s failed.\n", iepgSetup.sImageDirectory.c_str() );

		        fclose ( urlFile );

                        return false;
                    }
                } else
		{
		    closedir(tmpdir);
		}

                /*get EPG from network */
                if ( strlen ( url_str ) > 0 )
                    sleep ( 2 );              // avoid overloading server
                // also some websites impose a 2 sec gap between fetches


                if ( strlen ( url_str ) > 0 )
                {
                    if ( p != NULL & Download )  // Flag download from caller set, so download the file
                    {
                        strcpy(line, url_str);
                        // supply url-string with pin and / or day of desired epg - if configured
                        if ( strstr ( line, "%s" ) != NULL && strstr ( line, "%d" ) != NULL )
                            sprintf ( line, url_str, iepgSetup.SetupProviders[nProvider].Pin, WorkingDay );
                        else if ( strstr ( line, "%s" ) != NULL )
                            sprintf ( line, url_str, iepgSetup.SetupProviders[nProvider].Pin );
                        else if ( strstr ( line, "%d" ) != NULL )
                            sprintf ( line, url_str, WorkingDay );

// Update EpgFileName for subsequent EPG loads
// All Channel data are constant, but the filename need to reflect the day
                        if (ChannelList_Complete)
                            Urls->at(i).EpgFileName = tmp_EpgFileName;

                        if ( p->downloadURL ( /* url_str */  line, tmp_EpgFileName ) ) // downloadURL is member of class cThreadXmlEpg2vdr
                        {
                            dsyslog ( "iepg: getting URL for %s : %s", channelName_str, line );
                            char cmd_unzip[200];

                            // In case of provider type 1 unzip archive
                            if ( iepgSetup.SetupProviders[nProvider].ProviderType == 1 )
                            {
                                char *d = get_dir_per_day ( sDir, WorkingDay );

//                                        sprintf ( cmd_unzip, "mkdir %s;unzip %s -d %s", d, tmp_EpgFileName, d );
                                sprintf ( cmd_unzip, "unzip -o %s -d %s", tmp_EpgFileName, d );
                                SystemExec ( cmd_unzip );

                                strcpy ( sDir, d );
                                free ( d );

                                char *file = get_xml_from_zip ( sDir );

                                if ( file )
                                {
                                    tmp_obj.EpgFileName = file;
                                    free ( file );
                                }
                                else
                                    tmp_obj.EpgFileName = "";   // file was not found

                                if ( tmp_obj.EpgFileName.size() == 0 )
                                {
                                    tmp_obj.isError = true;
                                    esyslog ( "iepg: xml file in %s not found", sDir );
                                }
                            }
                            tmp_obj.isError = false;
                        }
                        else
                        {
                            tmp_obj.isError = true;
                            esyslog ( "iepg: Error downloading URL %s",url_str );

                            do_log ( sDir, LOGFILE_STATUS, tr ( "Error downloading URL" ), false );
                            do_log ( iepgSetup.sDataDirectory.c_str(), LOGFILE_THREAD, tr ( "Error downloading URL! Please check your PIN and the URL" ), false );

			    fclose ( urlFile );

                            return false;
                        }
                    }
                }
            }

            // Add to Url-List (Only the first day)
            if (!ChannelList_Complete)
                Urls->push_back ( tmp_obj );

            i++;
        }  // end if: data read
    }                          // end while: reading URL file and downloading URLs
	
    fclose ( urlFile );

    dsyslog ( "iepg: %s successfully read. %d entries", urlListFileName, i );


    return true;
}

int get_file_size(char *path)
{
    FILE *testFile;
    int lSize;

    testFile = fopen ( path, "r" );
    if ( testFile == NULL )
        return -1;


    fseek (testFile , 0 , SEEK_END);
    lSize = ftell (testFile);
    fclose (testFile);

    return lSize;
}

