/*
*  C Implementation: pngutil.c
*
* Description: A class to support read/write/operate with png images
*
*
* Author: RealQuick <realquick@doublequick.de>, (C) 2009
*
* Copyright: See COPYING file that comes with this distribution
*
*/

#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>


#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <vdr/osdbase.h>
#include <vdr/config.h>
#include "pngutils.h"
#include "i18n.h"
#include <string.h>
#include <string>

#include <png.h>
#include <jpeglib.h>

#include "common.h"


cPngUtils::cPngUtils()
{
    bmp = NULL;
}


cPngUtils::cPngUtils ( int w, int h )
{
    nMaxPNGWidth = width = w;
    nMaxPNGHeight = height = h;

    bmp = NULL;
}

cPngUtils::~cPngUtils()
{
    if ( bmp != NULL )
    {
        delete [] bmp;
    }
}


int cPngUtils::ReadJpeg ( char *sPath )
{
//  jpeg_destination_mgr jdm;

    unsigned char *linha;
    int ImageSize;

    struct jpeg_decompress_struct cinfo;
    struct jpeg_error_mgr jerr;

    FILE * infile;

    if ( ( infile = fopen ( sPath, "rb" ) ) == NULL )
    {
        esyslog ( "can't open %s\n", sPath );
        return ( -1 );
    }

    cinfo.err = jpeg_std_error ( &jerr );
    jpeg_create_decompress ( &cinfo );

    jpeg_stdio_src ( &cinfo, infile );
    jpeg_read_header ( &cinfo, TRUE );

    jpeg_start_decompress ( &cinfo );

    rowsize = cinfo.image_width * cinfo.out_color_components;
    ImageSize = cinfo.image_height * rowsize;

    // resolution has changed -> resize array
//    if(!(width == cinfo.image_width &&
//       height == cinfo.image_height))
    {
        if ( bmp != NULL )
        {
            delete [] bmp;
            bmp = NULL;
        }
    }

    if (bmp == NULL)
        bmp = new unsigned char[ImageSize];


    width = cinfo.image_width;
    height = cinfo.image_height;

    linha=bmp;
    for ( unsigned int i=0;i < cinfo.output_height; i++ )
    {
        linha = bmp + ( i * rowsize );
        jpeg_read_scanlines ( &cinfo,&linha,1 );
    }

    jpeg_finish_decompress ( &cinfo );
    jpeg_destroy_decompress ( &cinfo );

    fclose(infile);

    return 0;
}

int cPngUtils::WriteFile ( cOsd *Osd, unsigned char nTransparenz, int w, int h, int x, int y, int *nImageIdPos, bool bIsPNG )
{

    int xpos=0;
    int ypos=0;

    char sPath[512];

    if ( bmp == NULL )
        return -1;

    while ( ypos < h )
    {
        while ( xpos < w )
        {
            sprintf ( sPath, "/tmp/pic%d.png", ( *nImageIdPos ) %10 );
            int rc = WriteFile ( nTransparenz, xpos, ypos, sPath, bIsPNG , min ( nMaxPNGWidth, w-xpos ), min ( nMaxPNGHeight , h-ypos ), w * bIsPNG ? 3 : 4);

            if ( rc != 0 )
                return rc;
            Osd->SetImagePath ( HDOSD_MAX_IMAGES-11 + ( *nImageIdPos ) %10, sPath );
            Osd->DrawImage ( HDOSD_MAX_IMAGES-11 + ( *nImageIdPos ) %10, x+xpos, y+ypos, false );

            ( *nImageIdPos ) ++;
            xpos += nMaxPNGWidth;
        }
        ypos += nMaxPNGHeight ;
        xpos = 0;
    }

    return 0;
}


int cPngUtils::WriteFile ( unsigned char nTransparenz, int xpos, int ypos, char *sPath, bool bIsPNG, int rel_width, int rel_height, int rel_rowsize )
{
    png_structp  png_ptr;       /* note:  temporary variabloutfile = fopen(sPath, "wb");es! */
    png_infop  info_ptr;
    int color_type, interlace_type;

    int offset;

    int x, y;

    int nWidth, nHeight, nRowsize;

    // use relative width instead of real image with?
    if (rel_width != 0)
        nWidth = rel_width;
    else
        nWidth = width;

    // use relative height instead of real image height?
    if (rel_height != 0)
        nHeight = rel_height;
    else
        nHeight = height;

    // use relative rowsize instead of real image rowsize?
    if (rel_rowsize != 0)
        nRowsize = rel_rowsize;
    else
        nRowsize = rowsize;


    FILE *outfile;

    png_bytep *row_pointers;
    png_bytep *row_in_pointers;

    if ( bmp == NULL )
        return -1;

    /* could also replace libpng warning-handler (final NULL), but no need: */

    png_ptr = png_create_write_struct ( PNG_LIBPNG_VER_STRING, NULL, NULL, NULL );
    if ( !png_ptr )
        return 4;   /* out of memory */

    info_ptr = png_create_info_struct ( png_ptr );
    if ( !info_ptr )
    {
        png_destroy_write_struct ( &png_ptr, NULL );
        return 4;   /* out of memory */
    }


    outfile = fopen ( sPath, "wb" );
    if (outfile == NULL)
    {
        png_destroy_info_struct ( png_ptr, &info_ptr );
        png_destroy_write_struct ( &png_ptr, NULL );
        return 10;
    }

    if ( setjmp ( png_jmpbuf ( png_ptr ) ) )
    {
        png_destroy_info_struct ( png_ptr, &info_ptr );
        png_destroy_write_struct ( &png_ptr, NULL );
        fclose ( outfile );
        return 5;
    }
    png_init_io ( png_ptr, outfile );

    /* set the compression levels--in general, always want to leave filtering
     * turned on (except for palette images) and allow all of the filters,
     * which is the default; want 32K zlib window, unless entire image buffer
     * is 16K or smaller (unknown here)--also the default; usually want max
     * compression (NOT the default); and remaining compression flags should
     * be left alone */

    if ( setjmp ( png_jmpbuf ( png_ptr ) ) )
    {
        png_destroy_info_struct ( png_ptr, &info_ptr );
        png_destroy_write_struct ( &png_ptr, NULL );
        fclose ( outfile );
        return 6;
    }
//    png_set_compression_level ( png_ptr, Z_BEST_COMPRESSION );

    // performace issues MORE speed less compression
    png_set_filter(png_ptr, PNG_FILTER_TYPE_BASE, PNG_FILTER_SUB);
    png_set_compression_level(png_ptr, Z_BEST_SPEED);
    png_set_compression_strategy(png_ptr, Z_HUFFMAN_ONLY);

    color_type = PNG_COLOR_TYPE_RGB_ALPHA;
    interlace_type = PNG_INTERLACE_NONE;

    if ( setjmp ( png_jmpbuf ( png_ptr ) ) )
    {
        png_destroy_info_struct ( png_ptr, &info_ptr );
        png_destroy_write_struct ( &png_ptr, NULL );
        fclose ( outfile );
        return 7;
    }

    png_set_IHDR ( png_ptr, info_ptr, nWidth, nHeight,
                   /* sample_depth */ 8, color_type, interlace_type,
                   PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT );

//                case PNG_COLOR_TYPE_RGB_ALPHA: puts("color_type: PNG_COLOR_TYPE_RGB_ALPHA"); break;
    png_write_info ( png_ptr, info_ptr );

    row_pointers = ( png_bytep* ) malloc ( sizeof ( png_bytep ) * nHeight );
    for ( y=0; y<nHeight; y++ )
        row_pointers[y] = ( png_byte* ) malloc ( info_ptr->rowbytes );

    png_byte* row;
    png_byte* ptr;

    png_byte* row_in;
    png_byte* ptr_in;


    if ( bIsPNG )
    {
// esyslog("quickbrowser: offset=%d W=%d,h=%d,rowsize=%d,  xpos=%d,ypos=%d", offset, w,h,rowsize,xpos,ypos);

        offset = nMaxPNGWidth * ( xpos / nMaxPNGWidth ) * 4;
        row_in_pointers = ( png_bytep * ) bmp;
        for ( y=0; y<nHeight; y++ )
        {
            row = row_pointers[y];
            row_in = row_in_pointers[y+ypos];
            for ( x=0; x<nWidth; x++ )
            {
                ptr = & ( row[x*4] );
                ptr_in = & ( row_in[ ( x+xpos ) *4] );

                ptr[0] = ptr_in[0];
                ptr[1] = ptr_in[1];
                ptr[2] = ptr_in[2];
                ptr[3] = ptr_in[3];
            }
        }
    }
    else
    {
        // Copy and convert pixel  max nMaxPNGWidth * nMaxPNGHeight
        offset = nMaxPNGWidth * ( xpos / nMaxPNGWidth ) * 3;
        for ( y=0; y<nHeight; y++ )
        {
            row = row_pointers[y];
            for ( x=0; x<nWidth; x++ )
            {
                png_byte* ptr = & ( row[x*4] );

                ptr[0] = * ( bmp + ( y + ypos ) * nRowsize + 3*x + offset );
                ptr[1] = * ( bmp + ( y + ypos ) * nRowsize + 3*x + offset + 1 );
                ptr[2] = * ( bmp + ( y + ypos ) * nRowsize + 3*x + offset + 2 );
                ptr[3] = nTransparenz;
            }
        }
    }


    if ( setjmp ( png_jmpbuf ( png_ptr ) ) )
    {
        png_destroy_info_struct ( png_ptr, &info_ptr );
        png_destroy_write_struct ( &png_ptr, NULL );
        fclose ( outfile );
        return 8;
    }

    png_write_image ( png_ptr, row_pointers );

    /* since that's it, we also close out the end of the PNG file now--if we
     * had any text or time info to write after the IDATs, second argument
     * would be info_ptr, but we optimize slightly by sending NULL pointer: */

    if ( setjmp ( png_jmpbuf ( png_ptr ) ) )
    {
        png_destroy_info_struct ( png_ptr, &info_ptr );
        png_destroy_write_struct ( &png_ptr, NULL );
        fclose ( outfile );
        return 9;
    }
    png_write_end ( png_ptr, NULL );

    fclose ( outfile );

    /* cleanup heap allocation */
    for ( y=0; y<nHeight; y++ )
        free ( row_pointers[y] );
    free ( row_pointers );

    png_destroy_info_struct ( png_ptr, &info_ptr );
    png_destroy_write_struct ( &png_ptr, NULL );

    return 0;
}


int cPngUtils::DumpRaw ( char *sPath, int step )
{
    FILE *outfile;
    long nPixel;

    if ( ( outfile = fopen ( sPath, "wb" ) ) == NULL )
    {
        esyslog ( "can't open %s\n", sPath );
        return ( -1 );
    }

    if ( bmp == NULL )
        return -2;

    for ( unsigned int y=0;y<height;y+=step )
        for ( unsigned int x=0;x<width;x+=step )
        {
            png_byte* ptr = ( png_byte* ) &nPixel;

            ptr[2] = * ( bmp + ( y ) * rowsize + 3*x );
            ptr[1] = * ( bmp + ( y ) * rowsize + 3*x + 1 );
            ptr[0] = * ( bmp + ( y ) * rowsize + 3*x + 2 );
            ptr[3] = 250 /* nTransparenz */;
            fwrite ( &nPixel, sizeof ( nPixel ), 1, outfile );


        }

    fclose ( outfile );
    return 0;
}
