/*
*  C Implementation: setup.c
*/
// Description: Implementation for setup
//
//
// Author: RealQuick <RealQuick@doubleQuick.de>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <vdr/plugin.h>
#include <vdr/menuitems.h>

#include "setup.h"
#include "i18n.h"

#include "channelmap.h"
#include "common.h"

extern std::vector < cProviderStatus > Providers;

ciepgSetup iepgSetup;

ciepgSetup::ciepgSetup(void)
{
}

ciepgMenuSetupPage::ciepgMenuSetupPage(void)
{
    SetHelp(tr("Mapping"));

    m_NewSetup = iepgSetup;

//  m_NewSetup.Priority = 1;

    prio[0] = tr("High");
    prio[1] = tr("Medium");
    prio[2] = tr("Low");
    prio[3] = tr("Very low");

    noepg_mode[0] = tr("Off");
    noepg_mode[1] = tr("Housekeeping");
    noepg_mode[2] = tr("ChannelSwitch");
    noepg_mode[3] = tr("Housekeeping/Channelswitch");
    noepg_mode[4] = tr("NOEEPG-patched vdr");


    starttimes[0] = (char *)tr("Always manually");
    starttimes[1] = (char *)tr("10 min after startup");
    for (int i=2;i<=STARTTIME_VALUES;i++)
    {
        char dat[10];
        sprintf(dat, "%02d:00", i-1);
        starttimes[i] = strdup(dat);
    }

    Setup();
}

void ciepgMenuSetupPage::Setup(void)
{
    unsigned int nProvider = 0;
    cProviderStatus p;

    m_NewSetup.lastupdate = 0;
//  m_NewSetup.NoEPGWorkaround = 0;

    int current = Current();
    Clear();

    Add(new cMenuEditStraItem(tr("Priority"), &m_NewSetup.Priority, 4, prio));
    Add(new cMenuEditBoolItem(  tr("Use HTTP Proxy"), &m_NewSetup.useproxy, tr("no"), tr("yes")));
    if (m_NewSetup.useproxy)
        Add(new cMenuEditStrItem( tr("  HTTP Proxy"), m_NewSetup.httpproxy, sizeof(m_NewSetup.httpproxy), tr(FileNameChars)));

    Add(new cMenuEditStraItem(tr("Startup time"), &m_NewSetup.hour_to_start, 26, starttimes));

    Add(new cOsdItem("", osUnknown, false), false, NULL);

    Add(new cMenuEditBoolItem(tr("Convert pictures to png (for recordings)"), &m_NewSetup.convert_to_png, tr("no"), tr("yes")));
    Add(new cMenuEditBoolItem(tr("Save epg pictures to recording dir"), &m_NewSetup.save_rec_pic, tr("no"), tr("yes")));
    if (m_NewSetup.save_rec_pic)
        Add(new cMenuEditBoolItem( tr("  Only primetime events"), &m_NewSetup.pics_only_primetime, tr("no"), tr("yes")));

    Add(new cOsdItem("", osUnknown, false), false, NULL);
    Add(new cMenuEditBoolItem(tr("Convert pictures to raw (for epg view)"), &m_NewSetup.convert_to_raw, tr("no"), tr("yes")));

    /*
      Add(new cMenuEditBoolItem(tr("Avoid duplicate EPG entries (NoEPG-Workaround)"), &m_NewSetup.NoEPGWorkaround, tr("no"), tr("yes")));
    */
    Add(new cMenuEditStraItem(tr("Avoid duplicate EPG entries"), &m_NewSetup.NoEPGWorkaround, 5, noepg_mode));

    Add(new cOsdItem("", osUnknown, false), false, NULL);

//    m_NewSetup.SetupProviders.clear();	// reset list

    while (nProvider < Providers.size())
    {
        if (strlen(m_NewSetup.SetupProviders[nProvider].baseurl) == 0)
            switch (nProvider)
            {
            case 0:
                strcpy(m_NewSetup.SetupProviders[nProvider].baseurl,  BASE_URL_PROVIDERTYPE0);
                break;
            case 1:
                strcpy(m_NewSetup.SetupProviders[nProvider].baseurl, BASE_URL_PROVIDERTYPE1);
                break;
            }

        char sTitle[100];
        sprintf(sTitle, " ---- %s%s", tr("Settings for provider ---- "), Providers[nProvider].Name.c_str ());
        Add(new cOsdItem(sTitle, osUnknown, false), false, NULL);

        /*
           sprintf (provStr, "%s ", Providers[nProvider].Name.c_str ());
           Add (new cMenuEditBoolItem (provStr, &Providers[nProvider].Enabled, tr("Off"), tr("On")));
        */

	if(m_NewSetup.SetupProviders.size() <= nProvider)
	{
            p.Name=Providers[nProvider].Name;
            m_NewSetup.SetupProviders.push_back(p);
	}

        Add(new cMenuEditStrItem(tr("   Base Url"), m_NewSetup.SetupProviders[nProvider].baseurl, 500 /* , tr(FileNameChars) */));
        Add(new cMenuEditBoolItem(  tr("   Auto Update"), &m_NewSetup.SetupProviders[nProvider].autoupdate));
// do not provide periodically updates
//     Add(new cMenuEditIntItem(   tr("   Updatetime (min)"), &m_NewSetup.SetupProviders[nProvider].updatetime));
        Add(new cMenuEditIntItem(   tr("   Days in advance"), &m_NewSetup.SetupProviders[nProvider].days));
        Add(new cMenuEditStrItem(tr("   Pin"), m_NewSetup.SetupProviders[nProvider].Pin, 50, tr(FileNameChars)));

        Add(new cOsdItem("", osUnknown, false), false, NULL);

        nProvider++;
    }

    SetCurrent(Get(current));
    Display();

}

ciepgMenuSetupPage::~ciepgMenuSetupPage()
{
    // free memory allocated with strdup
    for (int i=2;i<=STARTTIME_VALUES;i++)
    {
        free (starttimes[i]);
    }
}

void ciepgMenuSetupPage::Store(void)
{
    SetupStore("UseProxy", m_NewSetup.useproxy);
    SetupStore("HTTPProxy", m_NewSetup.httpproxy);
    SetupStore("StartupTime", m_NewSetup.hour_to_start);
    SetupStore("LastUpdate", m_NewSetup.lastupdate);
    SetupStore("ConvertPng", m_NewSetup.convert_to_png);
    SetupStore("SavePictureRecordings", m_NewSetup.save_rec_pic);
    SetupStore("ConvertRaw", m_NewSetup.convert_to_raw);
    SetupStore("Priority", m_NewSetup.Priority);
    SetupStore("PicturesPrimeTime", m_NewSetup.pics_only_primetime);

    SetupStore("NoEPGWorkaround", m_NewSetup.NoEPGWorkaround);

    char sDummy[30];
    for (unsigned int i=0;i < Providers.size();i++)
    {
        switch (i)
        {
        case 0:
            m_NewSetup.SetupProviders[i].ProviderType = 0;
            break;
        case 1:
            m_NewSetup.SetupProviders[i].ProviderType = 1;
            break;
        }

        sprintf(sDummy,"%s%d", "BaseURL", i);
        SetupStore(sDummy, m_NewSetup.SetupProviders[i].baseurl);
        sprintf(sDummy,"%s%d", "Pin", i);
        SetupStore(sDummy, m_NewSetup.SetupProviders[i].Pin);
        sprintf(sDummy,"%s%d", "AutoUpdate", i);
        SetupStore(sDummy, m_NewSetup.SetupProviders[i].autoupdate);
        sprintf(sDummy,"%s%d", "DaysInAdvance", i);
        SetupStore(sDummy, m_NewSetup.SetupProviders[i].days);
        sprintf(sDummy,"%s%d", "ProviderType", i);
        SetupStore(sDummy, m_NewSetup.SetupProviders[i].ProviderType);
    }

    iepgSetup = m_NewSetup;
}

eOSState ciepgMenuSetupPage::ProcessKey ( eKeys Key )
{
    eOSState state = cMenuSetupPage::ProcessKey(Key);

    if ( state == osUnknown )
    {
        switch ( Key )
        {
        case kRed:
            Skins.Message(mtInfo, tr("reading actual configuration. Please wait..."), true);
            AddSubMenu(new cMenuChannelsMapping());
            break;
        default:
            break;
        }
        state = osContinue;
    }
    return state;
}
